# Rekall Memory Forensics
#
# Copyright 2013 Google Inc. All Rights Reserved.
#
# Authors:
# Michael Cohen <scudette@users.sourceforge.net>
# Michael Hale Ligh <michael.hale@gmail.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or (at
# your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

"""These are various utilities for rekall."""
import __builtin__
import cPickle
import cStringIO
import importlib
import itertools
import json
import ntpath
import re
import shutil
import socket
import sortedcontainers
import sys
import tempfile
import threading
import traceback
import time
import weakref


def SmartStr(string, encoding="utf8"):
    """Forces the string to be an encoded byte string."""
    if type(string) == unicode:
        return string.encode("utf8", "ignore")

    try:
        return string.__unicode__().encode(encoding)
    except AttributeError:
        return str(string)


def SmartUnicode(string, encoding="utf8"):
    """Forces the string into a unicode object."""
    try:
        # Allow the object to have a __unicode__ method.
        return unicode(string)
    except UnicodeError:
        return str(string).decode(encoding, "ignore")


def Hexdump(data, width=16):
    """Hexdump function shared by various plugins """
    for offset in xrange(0, len(data), width):
        row_data = data[offset:offset + width]
        translated_data = [
            x if ord(x) < 127 and ord(x) > 32 else "." for x in row_data]
        hexdata = " ".join(["{0:02x}".format(ord(x)) for x in row_data])

        yield offset, hexdata, translated_data


def WriteHexdump(renderer, data, base=0, width=16):
    """Write the hexdump to the fd."""
    renderer.table_header([dict(name='Offset', cname="offset", style="address"),
                           dict(name='Hex', cname="hex", width=width * 3),
                           dict(name='Data', cname='data', width=width)])

    for offset, hexdata, translated_data in Hexdump(data):
        renderer.table_row(base + offset, hexdata, "".join(translated_data))


# This is a synchronize decorator.
def Synchronized(f):
    """Synchronization decorator."""

    def NewFunction(self, *args, **kw):
        if self.lock:
            with self.lock:
                return f(self, *args, **kw)
        else:
            return f(self, *args, **kw)

    return NewFunction


class Node(object):
    """An entry to a linked list."""
    next = None
    prev = None
    data = None

    def __init__(self, data):
        self.data = data

    def __str__(self):
        return "Node:" + SmartStr(self.data)


class LinkedList(object):
    """A simple doubly linked list used for fast caches."""

    def __init__(self):
        # We are the head node.
        self.next = self.prev = self
        self.size = 0
        self.lock = threading.RLock()

    def Append(self, data):
        return self.AppendNode(Node(data))

    def AppendNode(self, node):
        self.size += 1
        last_node = self.prev

        last_node.next = node
        node.prev = last_node
        node.next = self
        self.prev = node

        return node

    def PopLeft(self):
        """Returns the head node and removes it from the list."""
        if self.next is self:
            raise IndexError("Pop from empty list.")

        first_node = self.next
        self.Unlink(first_node)

        return first_node.data

    def Pop(self):
        """Returns the tail node and removes it from the list."""
        if self.prev is self:
            raise IndexError("Pop from empty list.")

        last_node = self.tail
        self.Unlink(last_node)

        return last_node.data

    def Unlink(self, node):
        """Removes a given node from the list."""
        self.size -= 1

        node.prev.next = node.next
        node.next.prev = node.prev
        node.next = node.prev = None

    def __iter__(self):
        p = self.next
        while p is not self:
            yield p.data
            p = p.next

    def __len__(self):
        return self.size

    def __str__(self):
        p = self.next
        s = []
        while p is not self:
            s.append(str(p.data))
            p = p.next

        return "[" + ", ".join(s) + "]"


class FastStore(object):
    """This is a cache which expires objects in oldest first manner.

    This implementation first appeared in PyFlag and refined in GRR.

    This class implements an LRU cache which needs fast updates of the LRU order
    for random elements. This is implemented by using a dict for fast lookups
    and a linked list for quick deletions / insertions.
    """

    STORES = {}

    def __init__(self, max_size=10, kill_cb=None, lock=False):
        """Constructor.

        Args:
             max_size: The maximum number of objects held in cache.
             kill_cb: An optional function which will be called on each
                                object terminated from cache.
             lock: If True this cache will be thread safe.
        """
        self._age = LinkedList()
        self._hash = {}
        self._limit = max_size
        self._kill_cb = kill_cb
        self.lock = None
        if lock:
            self.lock = threading.RLock()
        self.hits = self.misses = 0
        self.creator = GetStack()
        self.STORES[id(self)] = weakref.proxy(
            self, lambda _, id=id(self), s=self.STORES: s.pop(id))

    def __len__(self):
        return len(self._hash)

    @Synchronized
    def Expire(self):
        """Expires old cache entries."""
        while len(self._age) > self._limit:
            x = self._age.PopLeft()
            self.ExpireObject(x)

    @Synchronized
    def Put(self, key, item):
        """Add the object to the cache."""
        hit = self._hash.get(key, self)
        if hit is not self:
            self._age.Unlink(hit[0])

        node = self._age.Append(key)
        self._hash[key] = (node, item)

        self.Expire()

        return key

    @Synchronized
    def ExpireObject(self, key):
        """Expire a specific object from cache."""
        _, item = self._hash.pop(key, (None, None))

        if self._kill_cb and item is not None:
            self._kill_cb(item)

        return item

    @Synchronized
    def ExpireRegEx(self, regex):
        """Expire all the objects with the key matching the regex."""
        reg = re.compile(regex)
        for key in self._hash.keys():
            if reg.match(key):
                self.ExpireObject(key)

    @Synchronized
    def ExpirePrefix(self, prefix):
        """Expire all the objects with the key having a given prefix."""
        for key in self._hash.keys():
            if key.startswith(prefix):
                self.ExpireObject(key)

    @Synchronized
    def Get(self, key):
        """Fetch the object from cache.

        Objects may be flushed from cache at any time. Callers must always
        handle the possibility of KeyError raised here.

        Args:
            key: The key used to access the object.

        Returns:
            Cached object.

        Raises:
            KeyError: If the object is not present in the cache.
        """
        hit = self._hash.get(key, self)
        if hit is self:
            self.misses += 1
            raise KeyError(key)

        # Remove the item and put to the end of the age list
        node, item = hit
        self._age.Unlink(node)
        self._age.AppendNode(node)
        self.hits += 1

        return item

    def __iter__(self):
        return self._hash.iteritems()

    def keys(self):
        return self._hash.keys()

    @Synchronized
    def __contains__(self, key):
        result = key in self._hash
        if result:
            node, _ = self._hash[key]
            self._age.Unlink(node)
            self._age.AppendNode(node)

        return result

    @Synchronized
    def __getitem__(self, key):
        return self.Get(key)

    @Synchronized
    def Flush(self):
        """Flush all items from cache."""
        while self._age:
            x = self._age.PopLeft()
            self.ExpireObject(x)

        self._hash = {}

    @Synchronized
    def __getstate__(self):
        """When pickled the cache is fushed."""
        if self._kill_cb:
            raise RuntimeError("Unable to pickle a store with a kill callback.")

        self.Flush()
        return dict(max_size=self._limit)

    def __setstate__(self, state):
        self.__init__(max_size=state["max_size"])


class AgeBasedCache(FastStore):
    """A cache which removes objects after some time."""

    def __init__(self, max_age=20, **kwargs):
        super(AgeBasedCache, self).__init__(**kwargs)
        self.max_age = max_age

    def Put(self, key, item):
        super(AgeBasedCache, self).Put(key, (item, time.time()))

    def Get(self, key):
        item, timestamp = super(AgeBasedCache, self).Get(key)

        if timestamp + self.max_age > time.time():
            return item

        else:
            self.ExpireObject(key)
            raise KeyError("Item too old.")


# Compensate for Windows python not supporting socket.inet_ntop and some
# Linux systems (i.e. OpenSuSE 11.2 w/ Python 2.6) not supporting IPv6.

def inet_ntop(address_family, packed_ip):

    def inet_ntop4(packed_ip):
        if not isinstance(packed_ip, str):
            raise TypeError("must be string, not {0}".format(type(packed_ip)))

        if len(packed_ip) != 4:
            raise ValueError("invalid length of packed IP address string")

        return "{0}.{1}.{2}.{3}".format(*[ord(x) for x in packed_ip])

    def inet_ntop6(packed_ip):
        if not isinstance(packed_ip, str):
            raise TypeError("must be string, not {0}".format(type(packed_ip)))

        if len(packed_ip) != 16:
            raise ValueError("invalid length of packed IP address string")

        words = []
        for i in range(0, 16, 2):
            words.append((ord(packed_ip[i]) << 8) | ord(packed_ip[i + 1]))

        # Replace a run of 0x00s with None
        numlen = [(k, len(list(g))) for k, g in itertools.groupby(words)]
        max_zero_run = sorted(sorted(
            numlen, key=lambda x: x[1], reverse=True),
                              key=lambda x: x[0])[0]
        words = []
        for k, l in numlen:
            if (k == 0) and (l == max_zero_run[1]) and not None in words:
                words.append(None)
            else:
                for i in range(l):
                    words.append(k)

        # Handle encapsulated IPv4 addresses
        encapsulated = ""
        if (words[0] is None) and (len(words) == 3 or (
                len(words) == 4 and words[1] == 0xffff)):
            words = words[:-2]
            encapsulated = inet_ntop4(packed_ip[-4:])
        # If we start or end with None, then add an additional :
        if words[0] is None:
            words = [None] + words
        if words[-1] is None:
            words += [None]
        # Join up everything we've got using :s
        return (":".join(
            ["{0:x}".format(w) if w is not None else "" for w in words]) +
                encapsulated)

    if address_family == socket.AF_INET:
        return inet_ntop4(packed_ip)
    elif address_family == socket.AF_INET6:
        return inet_ntop6(packed_ip)
    raise socket.error("[Errno 97] Address family not supported by protocol")


def ConditionalImport(name):
    try:
        return importlib.import_module(name)
    except ImportError:
        pass


# This is only available on unix systems.
fcntl = ConditionalImport("fcntl")


class FileLock(object):
    """A self releasing file lock."""

    def __init__(self, fd):
        self.fd = fd

    def __enter__(self):
        if fcntl:
            fcntl.flock(self.fd.fileno(), fcntl.LOCK_EX)
        return self.fd

    def __exit__(self, exc_type, exc_value, traceback):
        if fcntl:
            fcntl.flock(self.fd.fileno(), fcntl.LOCK_UN)


class TempDirectory(object):
    """A self cleaning temporary directory."""

    def __enter__(self):
        self.name = tempfile.mkdtemp()

        return self.name

    def __exit__(self, exc_type, exc_value, traceback):
        shutil.rmtree(self.name, True)


class AttributedString(object):
    """This is just a container for a string and some metadata."""
    highlights = None

    def __init__(self, value, highlights=None):
        self.highlights = highlights
        self.value = value

    def __unicode__(self):
        return unicode(self.value)

    def __str__(self):
        return str(self.value)

    def __repr__(self):
        return "AttributedString(%s, highlights=%s)" % (repr(self.value),
                                                        repr(self.highlights))


class HexDumpedString(AttributedString):
    """A string which should be hex dumped."""


class AttributeDict(dict):
    """A dict that can be accessed via attributes."""
    dirty = False

    def __setattr__(self, attr, value):
        try:
            # Check that the object itself has this attribute.
            object.__getattribute__(self, attr)

            return object.__setattr__(self, attr, value)
        except AttributeError:
            self.Set(attr, value)

    def Get(self, item, default=None):
        return self.get(item, default)

    def Set(self, attr, value):
        self.dirty = True

        # Setting a key to None means to remove it from the cache. NOTE! This
        # must be exactly None not a NoneObject() since it should be possible to
        # cache a NoneObject() as the value of some key.
        if value is None:
            self.pop(attr, None)
        else:
            self[attr] = value

    def __getattr__(self, attr):
        # Do not allow private attributes to be set.
        if attr.startswith("_"):
            raise AttributeError(attr)

        return self.get(attr)

    def __dir__(self):
        return sorted(self)


def FormatIPAddress(family, value):
    """Formats a value as an ascii IP address determined by family."""
    if value == None:
        return value

    return socket.inet_ntop(
        getattr(socket, str(family)),
        value.obj_vm.read(value.obj_offset, value.obj_size))


def ntoh(value):
    size = value.obj_size
    if size == 2:
        return socket.ntohs(value.v())
    elif size == 4:
        return socket.ntohl(value.v())

    from rekall import obj
    return obj.NoneObject("Not a valid integer")


def Invert(dictionary):
    """Inverts keys and values in dictionary.

    Assume the keys and values are unique.
    """
    return {v:k for k, v in dictionary.items()}


def PPrint(data, depth=0):
    """A pretty printer for a profile.

    This only supports dict, list and non-unicode strings.

    This produces both a valid json and a valid python file.
    """
    result = []
    if type(data) is bool:
        return str(data).lower()

    if isinstance(data, dict):
        # Empty dicts emitted on one line.
        if not data:
            return "{}"

        result.append("{")
        tmp = []
        for key, value in sorted(data.items()):
            # Only emit non-empty dicts.
            if value != {}:
                tmp.append(
                    " %s%s: %s" % (
                        " " * depth,
                        json.dumps(str(key)),
                        PPrint(data[key], depth + 1).strip()))

        result.append(", \n".join(tmp)[depth:])

        result.append("}")
        return "\n".join([(" " * depth + x) for x in result])

    if isinstance(data, (list, tuple)):
        for item in data:
            pp_item = PPrint(item, depth)

            result.append(pp_item.strip())

        res = "[" + ", ".join(result) + "]"

        return res

    if isinstance(data, basestring):
        return json.dumps(SmartUnicode(data))

    # JSON encodes None as null.
    elif data is None:
        return "null"

    return SmartStr(data)


DEFINE_REGEX = re.compile(r"#define\s+([A-Z0-9_]+)\s+((0x)?[0-9A-Z]+)")

def MaskMapFromDefines(text):
    """Generates a maskmap dict from a list of #defines.

    This function allows us to copy the relevant #define sections from header
    files without needing to manually edit them. We get to keep the comments etc
    for readability.
    """
    result = {}
    for line in text.splitlines():
        m = DEFINE_REGEX.search(line)
        if m:
            name = m.group(1)
            value = m.group(2)
            if m.group(3):
                value = int(value, 16)
            elif value.startswith("0"):
                value = int(value, 8)
            else:
                value = int(value)

            result[name] = value

    return result


def EnumerationFromDefines(text):
    """Generate an Enumeration from a list of #defines."""
    result = {}
    for line in text.splitlines():
        m = DEFINE_REGEX.search(line)
        if m:
            name = m.group(1)
            value = m.group(2)
            if m.group(3):
                value = int(value, 16)
            else:
                value = int(value)

            result[value] = name

    return result


class SortedCollection(sortedcontainers.SortedDict):
    def __init__(self, *args, **kwargs):
        self.key_func = kwargs.pop("key", lambda x: x[0])
        super(SortedCollection, self).__init__(*args, **kwargs)

    def insert(self, item):
        key = self.key_func(item)
        self[key] = item

    def __iter__(self):
        return self.itervalues()

    def get_value_smaller_than(self, k):
        for x in self.irange(0, k, reverse=True):
            return x, self[x]

        return None, None

    def get_value_larger_than(self, k):
        for x in self.irange(k):
            return x, self[x]

        return None, None

    def find_le(self, k):
        for x in self.irange(0, k, reverse=True):
            return self[x]

        raise ValueError('No item found with key below: %r' % (k,))

    def find_ge(self, k):
        for x in self.irange(k):
            return self[x]

        raise ValueError('No item found with key at or above: %r' % (k,))



class RangedCollection(object):
    """A convenience wrapper around SortedCollection for ranges."""

    def __init__(self):
        self.collection = SortedCollection()

    def insert(self, start, end, data):
        start = int(start)
        end = int(end)
        self.collection[(start, end)] = data

    def get_next_range_start(self, address):
        """Gets the start address of the next range larger than address."""
        range, _ = self.collection.get_value_larger_than((address, None))
        if range is not None:
            return range[0]

    def get_containing_range(self, address):
        """Retrieve the data associated with the range that contains value.

        Retuns:
          A tuple of start, end, data for the range that contains address.
        """
        tmp, data = self.collection.get_value_smaller_than((address + 1, None))
        if tmp is not None:
            start, end = tmp
            if start <= address < end:
                return start, end, data

        return None, None, None

    def __eq__(self, other):
        if isinstance(other, self.__class__):
            return self.collection == other.collection

        return NotImplemented

    def clear(self):
        self.collection.clear()

    def __getitem__(self, item):
        key = self.collection.iloc[item]
        return key[0], key[1], self.collection[key]

    def __iter__(self):
        for (start, end), data in self.collection.iteritems():
            yield start, end, data

    def __reversed__(self):
        for key in reversed(self.collection):
            start, end = key
            yield start, end, self.collection[key]

    def __str__(self):
        result = []
        for start, end, data in self:
            result.append("<%#x, %#x> %s" % (start, end, data))

        return "\n".join(result)


class JITIterator(object):
    def __init__(self, baseclass):
        self.baseclass = baseclass

    def __contains__(self, item):
        return item in list(self)

    def __str__(self):
        return str(list(self))

    def __iter__(self):
        return (
            x.name for x in self.baseclass.classes.values() if x.name)


def CopyFDs(in_fd, out_fd, length=2**64):
    """Copy from one fd to another.

    If length is specified, we stop when we copied this many bytes. We always
    stop when in_fd reaches EOF.
    """
    while length > 0:
        data = in_fd.read(min(10000000, length))
        if not data:
            return

        out_fd.write(data)
        length -= len(data)


def CopyAStoFD(in_as, out_fd, start=0, length=2**64,
               cb=lambda off, length: None):
    """Copy an address space into a file-like object."""
    blocksize = 1024 * 1024

    for run in in_as.get_address_ranges(start=start, end=start+length):
        for offset in xrange(run.start, run.end, blocksize):
            to_read = min(blocksize, run.end - offset, length)
            if to_read == 0:
                break

            data = in_as.read(offset, to_read)

            out_fd.seek(offset)
            out_fd.write(data)
            length -= len(data)

            cb(offset, len(data))


def issubclass(obj, cls):    # pylint: disable=redefined-builtin
    """A sane implementation of issubclass.

    See http://bugs.python.org/issue10569

    Python bare issubclass must be protected by an isinstance test first since
    it can only work on types and raises when provided something which is not a
    type.

    Args:
      obj: Any object or class.
      cls: The class to check against.

    Returns:
      True if obj is a subclass of cls and False otherwise.
    """
    return isinstance(obj, type) and __builtin__.issubclass(obj, cls)


def XOR(string1, string2):
    """Returns string1 xor string2."""
    return "".join(chr(ord(x) ^ ord(y)) for x, y in zip(string1, string2))


def xrange(start, end, step=1):
    """In Python2 the xrange builtin is broken.

    It raises when start or end do not fit in an int. Since python does not
    generally care about this we need to implement a clean version of this
    builtin.
    """
    x = start
    while x < end:
        yield x
        x += step


def SafePickle(data):
    """An unpickler for serialized tuple/lists/strings etc.

    Does not support recovering instances.
    """
    return cPickle.dumps(data, -1)

def SafeUnpickler(data):
    """An unpickler for serialized tuple/lists/strings etc.

    Does not support recovering instances.
    """
    unpickler = cPickle.Unpickler(cStringIO.StringIO(data))
    unpickler.find_global = None

    return unpickler.load()


def SplitPath(path):
    """Splits the path into a list of components."""
    result = []
    while 1:
        path, filename = ntpath.split(path)

        if filename:
            result.append(filename)
        else:
            if path and path not in ("\\", "/"):
                result.append(path)
            break

    result.reverse()
    return result


def CaseInsensitiveDictLookup(key, dictionary):
    """Lookup the dictionary with a case insensitive key."""
    # First try as is.
    result = dictionary.get(key)
    if result is None:
        for k, v in dictionary.iteritems():
            if k.lower() == key.lower():
                return v

    return result


def TimeIt(f):
    def NewFunction(self, *args, **kw):
        try:
            now = time.time()
            return f(self, *args, **kw)
        finally:
            print "Took %s sec" % (time.time() - now)

    return NewFunction

def GetStack():
    """Returns the current call stack as a string."""
    return "".join(traceback.format_stack())


def InternObject(obj):
    """Copies and interns strings in a recursive object."""
    obj_cls = obj.__class__
    if obj_cls is str:
        return intern(obj)

    if obj_cls is unicode:
        return intern(str(obj))

    if obj_cls is dict:
        result = {}
        for k, v in obj.iteritems():
            k = InternObject(k)
            v = InternObject(v)
            result[k] = v

        return result

    if obj_cls is list:
        return [InternObject(x) for x in obj]

    return obj


class safe_property(property):
    """Re-Raises AttributeError in properties.

    In Python @property swallows AttributeError and calls __getattr__. This is
    rarely what you want because sometime an AttributeError is erronously raised
    from legitimately broken property code and just swallowing it automatically
    can cause weird error messages (e.g. Attribute foobar does not exist, if
    foobar is a property) or even worse, it calls __getattr__ which does
    something completely different.
    """

    def __get__(self, *args, **kwargs):
        try:
            return super(safe_property, self).__get__(*args, **kwargs)
        except AttributeError as e:
            message = "AttributeError raised: %s" % e

            # Retain the original backtrace but re-raise a RuntimeError to
            # prevent the property from calling __getattr__.
            raise RuntimeError, message, sys.exc_info()[2]



# Rekall Memory Forensics
# Copyright 2013 Google Inc. All Rights Reserved.
#
# Author: Michael Cohen scudette@google.com
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or (at
# your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
#
import gzip
import os

from rekall import constants
from rekall import io_manager
from rekall import testlib


class IOManagerTest(testlib.RekallBaseUnitTestCase):
    """Test the IO Manager."""

    DATA = {
        "foo.gz": "hello",
        "bar": "goodbye"
    }

    def setUp(self):
        super(IOManagerTest, self).setUp()

        # Create a new repository in the temp directory.
        self.version = constants.PROFILE_REPOSITORY_VERSION
        for filename, data in self.DATA.iteritems():
            path = os.path.join(self.temp_directory, self.version,
                                filename)

            if path.endswith("gz"):
                opener = gzip.open
            else:
                opener = open

            try:
                os.makedirs(os.path.dirname(path))
            except (OSError, IOError):
                pass

            with opener(path, "wb") as fd:

                fd.write(data)

    def testDirectoryIOManager(self):
        manager = io_manager.DirectoryIOManager(
            self.temp_directory,
            session=self.MakeUserSession())

        # Cant decode from json.
        self.assertEqual(manager.GetData("foo"), None)
        self.assertEqual(manager.GetData("foo", raw=True),
                         "hello")

        # Test ListFiles().
        self.assertListEqual(sorted(manager.ListFiles()),
                             ["bar", "foo"])

        # Storing a data structure.
        data = dict(a=1)
        manager.StoreData("baz", data)
        self.assertDictEqual(manager.GetData("baz"),
                             data)

        self.assertTrue(
            isinstance(manager.GetData("baz", raw=True), basestring))



"""Tests for profile_tool."""

import logging
import unittest

from rekall import session
from rekall import testlib
from rekall.plugins.tools import profile_tool


class BuildIndexTester(profile_tool.BuildIndex):
    def __init__(self, profiles=None, **kwargs):
        super(BuildIndexTester, self).__init__(**kwargs)
        self.profiles = profiles

    def _LoadRawProfile(self, filepath):
        self.profiles[self._ProfileIdFromPath(filepath)]

    def _FindNewProfiles(self, *args, **kwargs):
        return self.profiles


class BuildSymbolOffsetIndexTest(testlib.RekallBaseUnitTestCase):
    def setUp(self):
        # This is the set of profiles we're trying to index. It covers all
        # the edge cases I've seen "in the wild" so far.
        self.raw_profiles = {
            # P1 is a profile that will have a single symbol trait = c:3 once
            # we remove duplicates.
            "P1": {
                "$CONSTANTS":
                {
                    "a": 1,
                    "b": 2,
                    "c": 3
                }
            },
            # P1-DUPLICATE is simply a duplicate profile from P1, to test that
            # we detect and discard profiles that are new and duplicates with
            # other new profiles.
            "P1-DUPLICATE": {
                "$CONSTANTS":
                {
                    "a": 1,
                    "b": 2,
                    "c": 3,
                }
            },
            # P1-1 is simply a newer version of P1. Simulates the usual
            # minor kernel version bump where some symbols are readjusted.
            # Single symbol traits = c:4
            "P1-1": {
                "$CONSTANTS":
                {
                    "a": 1,
                    "b": 2,
                    "c": 4
                }
            },

            # P1-2 simulates a slightly newer P1 profile that's virtually the
            # same but has an artifact of a static function that's been
            # chunked by GCC and its symbol made public.
            #
            # Real-world example:
            #   * Ubuntu precise 3.19.0-21-generic
            #   * Ubuntu precise 3.19.0-22-generic
            #
            # P1-2 should be considered a duplicate of P1.
            "P1-2": {
                        #
                        # function that's been chunked by GCC and its symbol
                        # made public.
                "$CONSTANTS":
                {
                    "a": 1,
                    "b": 2,
                    "c": 3,
                    "d.part.3": 4
                }
            },

            # P3 is a completely different profile.
            # Single symbol traits = b:3, c:5, d:6
            "P3": {
                "$CONSTANTS":
                {
                    "b": 3,
                    "c": 5,
                    "d": 6
                }
            },

            # Next is an example of profiles that collectively clash with
            # another profile. DOPPLE-1 and DOPPLE-2 make DOPPLE not have any
            # single-symbol trait. This forces us to compute traits with a
            # pair of symbols.
            #
            # Real-world example:
            #   * Ubuntu precise 3.2.0-72-generic
            #   * Ubuntu precise 3.2.0-73-generic
            #   * Ubuntu precise 3.2.0-74-generic
            # Single-symbol traits = d:9
            "DOPPLE-1": {
                "$CONSTANTS":
                {
                    "b": 7,
                    "c": 8,
                    "d": 9
                }
            },
            # The clashing profile.
            # Double-symbol trait = (b:7, d:10)
            "DOPPLE": {
                "$CONSTANTS":
                {
                    "b": 7,
                    "c": 8,
                    "d": 10
                }
            },
            # Single-symbol traits = b:1
            "DOPPLE-2": {
                "$CONSTANTS":
                {
                    "b": 1,
                    "c": 8,
                    "d": 10
                }
            },
        }

        self.build_index = BuildIndexTester(profiles=self.raw_profiles,
                                            session=session.Session())
        # A dict of profile_symbols.
        self.symbols_dict = dict([(x[0], x[1].get("$CONSTANTS"))
                                  for x in self.raw_profiles.iteritems()])

    def testFindTraits(self):
        traits = self.build_index._FindTraits(
            profile_id="P3",
            profiles=self.symbols_dict,
            num_traits=3, trait_length=1)
        self.assertListEqual(sorted(traits),
                             [
                                 [("b", 3)],
                                 [("c", 5)],
                                 [("d", 6)]
                             ])


        # We can't find any traits when there are duplicates
        traits = self.build_index._FindTraits(
            profile_id="P1",
            profiles=self.symbols_dict,
            num_traits=3, trait_length=1)
        self.assertListEqual(sorted(traits), [])

        # But if we remove duplicates, we find them.
        symbols_dict_nodups = self.symbols_dict.copy()
        symbols_dict_nodups.pop("P1-DUPLICATE")
        symbols_dict_nodups.pop("P1-2")
        traits = self.build_index._FindTraits(
            profile_id="P1",
            profiles=symbols_dict_nodups,
            num_traits=3, trait_length=1)
        self.assertListEqual(sorted(traits),
                             [
                                 [("c", 3)]
                             ])

        # Some profiles simply don't have single symbol traits..
        traits = self.build_index._FindTraits(
            profile_id="DOPPLE",
            profiles=self.symbols_dict,
            num_traits=3, trait_length=1)
        self.assertListEqual(sorted(traits), [])

        # But we can find 2-symbol traits.
        traits = self.build_index._FindTraits(
            profile_id="DOPPLE",
            profiles=self.symbols_dict,
            num_traits=3, trait_length=2)
        self.assertListEqual(sorted(traits),
                             [
                                 [("b", 7), ("d", 10)],
                             ])

    def testFindProfilesWithSymbolOffset(self):
        results = self.build_index._FindProfilesWithSymbolOffset(
            "b", 7, profiles=self.symbols_dict)
        self.assertEqual(results,
                         set(["DOPPLE-1", "DOPPLE"]))

        results = self.build_index._FindProfilesWithSymbolOffset(
            "NONEXISTING", 12, profiles=self.symbols_dict)
        self.assertEqual(results, set())



if __name__ == '__main__':
  logging.basicConfig(level=logging.DEBUG)
  unittest.main()



from rekall import addrspace
from rekall import testlib
from rekall import session


class CustomRunsAddressSpace(addrspace.RunBasedAddressSpace):
    def __init__(self, runs=None, data=None, **kwargs):
        super(CustomRunsAddressSpace, self).__init__(**kwargs)
        self.base = addrspace.BufferAddressSpace(data=data,
                                                 session=self.session)
        for i in runs:
            self.add_run(*i)


class RunBasedTest(testlib.RekallBaseUnitTestCase):
    """Test the RunBasedAddressSpace implementation."""

    def setUp(self):
        self.session = session.Session()
        self.test_as = CustomRunsAddressSpace(
            session=self.session,
            #        Voff, Poff, length
            runs=[(1000, 0, 10),    # This contains data.
                  (1020, 40, 10),
                  (1030, 50, 10),   # Contiguous runs.
                  (1050, 0, 2),
                  (1052, 5, 2)],
            data="0123456789")

    def testRunsRead(self):
        # Read from an address without data
        self.assertEqual(self.test_as.read(0, 20), "\x00" * 20)

        # Address translation.
        self.assertEqual(self.test_as.vtop(0), None)
        self.assertEqual(self.test_as.vtop(1005), 5)

        # Read spanning two runs
        self.assertEqual(self.test_as.read(1050, 4), "0156")

        # Read in the middle of a run
        self.assertEqual(self.test_as.read(1005, 10),
                         "56789" + "\x00" * 5)
        # Read past the end
        self.assertEqual(self.test_as.read(2000, 10),
                         "\x00" * 10)

    def testDiscontiguousRunsGetRanges(self):
        """Test the range merging."""
        runs = []
        # The pure mapping is not merged.
        for run in self.test_as.get_mappings():
            self.assertTrue(isinstance(run, addrspace.Run))
            self.assertEqual(run.address_space, self.test_as.base)
            runs.append([run.start, run.end, run.file_offset])

        self.assertEqual(runs,
                         [[1000, 1010, 0],
                          [1020, 1030, 40],
                          [1030, 1040, 50],
                          [1050, 1052, 0],
                          [1052, 1054, 5]])

        runs = []
        for run in self.test_as.merge_base_ranges():
            runs.append([run.start, run.end, run.file_offset])

        # merge_base_ranges is supposed to merge contiguous runs but still
        # maintain contiguous ranges in the base address space.
        self.assertEqual(runs,
                         [[1000, 1010, 0],
                          [1020, 1040, 40],
                          [1050, 1052, 0],
                          [1052, 1054, 5]])

        runs = []
        for run in self.test_as.get_address_ranges():
            # No valid physical mapping given here.
            self.assertTrue(run.file_offset is None)
            self.assertTrue(run.address_space is None)
            runs.append([run.start, run.end])

        # get_address_ranges is supposed to merge contiguous runs in the virtual
        # AS.
        self.assertEqual(runs,
                         [[1000, 1010],
                          [1020, 1040],
                          [1050, 1054]])


        # Check that get_address_ranges honors the start and end parameters.
        run = None
        for run in self.test_as.get_address_ranges(start=1022, end=1024):
            self.assertEqual(run.start, 1022)
            self.assertEqual(run.end, 1024)

            # get_address_ranges does not have a file_offset member.
            self.assertEqual(run.file_offset, None)

        self.assertTrue(run)


        # Check that merge_base_ranges honors the start and end parameters.
        run = None
        for run in self.test_as.merge_base_ranges(start=1022, end=1024):
            self.assertEqual(run.start, 1022)
            self.assertEqual(run.end, 1024)

            # The file_offset must be properly adjusted too.
            self.assertEqual(run.file_offset, 40 + 2)

        self.assertTrue(run)

        # Check that get_mappings honors the start parameter.
        run = None
        for run in self.test_as.get_mappings(start=1022):
            # get_mappings does not clip ranges so we may get a range which
            # starts below the specified limit.
            self.assertEqual(run.start, 1020)
            self.assertEqual(run.end, 1030)

            # The file_offset is unchanged.
            self.assertEqual(run.file_offset, 40)
            break

        self.assertTrue(run)



# Rekall Memory Forensics
# Copyright 2014 Google Inc. All Rights Reserved.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or (at
# your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
#

"""This module implements renderers specific to Linux structures."""

import os

from rekall.ui import json_renderer
from rekall.ui import text
from rekall.plugins.addrspaces import amd64
from rekall.plugins.renderers import base_objects
from rekall.plugins.renderers import data_export


class kuid_t_TextObjectRenderer(text.TextObjectRenderer):
    renders_type = "kuid_t"
    renderers = ["TextRenderer", "TestRenderer", "WideTextRenderer"]

    def render_row(self, target, **_):
        return text.Cell(unicode(target))

class kgid_t_TextObjectRenderer(kuid_t_TextObjectRenderer):
    renders_type = "kgid_t"


class kuid_t_JsonObjectRenderer(json_renderer.JsonObjectRenderer):
    renders_type = ["kuid_t", "kgid_t"]
    renderers = ["JsonRenderer", "DataExportRenderer"]

    def EncodeToJsonSafe(self, task, **_):
        return task.val.v()


class XenM2PMapperObjectRenderer(json_renderer.JsonObjectRenderer):
    renders_type = "XenM2PMapper"

    def EncodeToJsonSafe(self, item, **_):
        result = {}
        result["m2p_map"] = dict(item)
        result["mro"] = ":".join(self.get_mro(item))

        return result

    def DecodeFromJsonSafe(self, value, _):
        return amd64.XenM2PMapper(value["m2p_map"])


class TaskStruct_TextObjectRenderer(base_objects.StructTextRenderer):
    renders_type = "task_struct"
    COLUMNS = [
        dict(name="task_struct", style="address", cname="obj_offset"),
        dict(name="Name", width=20, align="l", cname="name"),
        dict(name="PID", width=6, align="r", cname="pid")
    ]


class TaskStruct_DataExport(data_export.DataExportBaseObjectRenderer):
    renders_type = "task_struct"

    def EncodeToJsonSafe(self, task, **_):
        result = super(TaskStruct_DataExport, self).EncodeToJsonSafe(task)
        fullpath = task.get_path(task.mm.m("exe_file"))
        result["Cybox"] = dict(
            type=u"ProcessObj:ProcessObjectType",
            Name=task.name,
            PID=task.pid,
            Creation_Time=task.task_start_time,
            Parent_PID=task.parent.pid,
            Image_Info=dict(
                type=u"ProcessObj:ImageInfoType",
                Path=fullpath,
                Command_Line=task.commandline,
                TrustedPath=fullpath,
                File_Name=os.path.basename(fullpath),
                )
            )

        res = json_renderer.JsonObjectRenderer.EncodeToJsonSafe(self, result)
        return res

    def Summary(self, item, **_):
        return "%s (%s)" % (item.get("Cybox", {}).get("Name", ""),
                            item.get("Cybox", {}).get("PID", ""))



# Rekall Memory Forensics
# Copyright 2014 Google Inc. All Rights Reserved.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or (at
# your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
#

"""Rekall's search function."""

__author__ = "Adam Sindelar <adamsh@google.com>"


from rekall import obj
from rekall import plugin
from rekall import testlib
from rekall import utils

from rekall.ui import identity as identity_renderer

from efilter import ast
from efilter import errors
from efilter import protocol
from efilter import query as q

from efilter.transforms import asdottysql
from efilter.transforms import solve
from efilter.transforms import infer_type

from efilter.protocols import applicative
from efilter.protocols import associative
from efilter.protocols import repeated
from efilter.protocols import structured


class TestWhichPlugin(testlib.SimpleTestCase):
    PLUGIN = "which_plugin"
    PARAMETERS = dict(
        commandline="which_plugin %(struct)s",
        struct="proc"
    )


class TestCollect(testlib.SimpleTestCase):
    PLUGIN = "collect"
    PARAMETERS = dict(
        commandline="collect %(struct)s",
        struct="proc"
    )


class TestExplain(testlib.SimpleTestCase):
    PLUGIN = "explain"
    PARAMETERS = dict(
        commandline="explain %(query)r",
        query="select * from pslist where (proc.pid == 1)"
    )


class TestSearch(testlib.SimpleTestCase):
    PLUGIN = "search"
    PARAMETERS = dict(
        commandline="search %(query)r",
        query="select * from pslist where (proc.pid == 1)"
    )


class FindPlugins(plugin.ProfileCommand):
    """Find which plugin(s) are available to produce the desired output."""

    name = "which_plugin"

    type_name = None
    producers_only = False

    @classmethod
    def args(cls, parser):
        super(FindPlugins, cls).args(parser)
        parser.add_argument("type_name", required=True,
                            help="The name of the type we're looking for. "
                                 "E.g.: 'proc' will find psxview, pslist, etc.")
        parser.add_argument("producers_only", required=False, default=False,
                            help="Only include producers: plugins that output "
                                 "only this struct and have no side effects.")

    def __init__(self, type_name=None, producers_only=False, **kwargs):
        super(FindPlugins, self).__init__(**kwargs)
        self.type_name = type_name
        self.producers_only = producers_only

    def collect(self):
        if self.producers_only:
            pertinent_cls = plugin.Producer
        else:
            pertinent_cls = plugin.TypedProfileCommand

        for plugin_cls in plugin.Command.classes.itervalues():
            if not plugin_cls.is_active(self.session):
                continue

            if not issubclass(plugin_cls, pertinent_cls):
                continue

            for t in plugin_cls.table_header.types_in_output:
                if isinstance(t, type) and self.type_name == t.__name__:
                    yield plugin_cls(session=self.session)
                elif self.type_name == t:
                    yield plugin_cls(session=self.session)

    def render(self, renderer):
        renderer.table_header([
            dict(name="Plugin", cname="plugin", type="Plugin", style="compact",
                 width=30)
        ])

        for command in self.collect():
            renderer.table_row(command)


class Collect(plugin.ProfileCommand):
    """Collect instances of struct of type 'type_name'.

    This plugin will find all other plugins that produce 'type_name' and merge
    all their output. For example, running collect 'proc' will give you a
    rudimentary psxview.

    This plugin is mostly used by other plugins, like netstat and psxview.

    """

    name = "collect"

    type_name = None

    @classmethod
    def args(cls, parser):
        super(Collect, cls).args(parser)
        parser.add_argument("type_name", required=True,
                            help="The type (struct) to collect.")

    @classmethod
    def GetPrototype(cls, session):
        """Instantiate with suitable default arguments."""
        return cls(None, session=session)

    def __init__(self, type_name, **kwargs):
        super(Collect, self).__init__(**kwargs)
        self.type_name = type_name

    def collect(self):
        which = self.session.plugins.which_plugin(type_name=self.type_name,
                                                  producers_only=True)

        results = {}
        for producer in which.collect():
            # We know the producer plugin implements 'produce' because
            # 'which_plugin' guarantees it.
            for result in producer.produce():
                previous = results.get(result.indices)
                if previous:
                    previous.obj_producers.add(producer.name)
                else:
                    result.obj_producers = set([producer.name])
                    results[result.indices] = result

        return results.itervalues()

    def render(self, renderer):
        renderer.table_header([
            dict(name=self.type_name, cname=self.type_name,
                 type=self.type_name),
            dict(name="Producers", cname="producers")
        ])

        for result in self.collect():
            renderer.table_row(result, result.obj_producers)


class CommandWrapper(object):
    """Wraps a plugin and its output for the purpose of EFILTER searches.

    This is a helper class for the Search plugin. It lets us pretend that
    plugins are functions to be called from inside EFILTER queries, and also
    takes care of running the plugin and saving its output and headers.

    Members:
        plugin_cls: The type of the Command subclass.
        rows: Output of rendering the plugin.
        columns: How 'rows' are structured.
        table_header: If Command is a subclass of TypedProfileCommand then this
            will contain its table header once applied.
    """
    plugin_cls = None
    rows = None
    columns = None
    table_header = None
    session = None

    # Once the CommandWrapper is run, this will be set to the arguments that
    # were used. You cannot apply the same CommandWrapper twice with different
    # args. If you need to do that, create two instances of CommandWrapper.
    _applied_args = None

    def __init__(self, plugin_cls, session):
        self.plugin_cls = plugin_cls
        self.session = session

    def __iter__(self):
        if self._applied_args is None:
            self.apply((), {})

        return iter(self.rows)

    def __getitem__(self, idx):
        if self._applied_args is None:
            self.apply((), {})

        return self.rows[idx]

    # IApplicative

    def apply(self, args, kwargs):
        """Instantiate the plugin with given args and run it.

        Note: Apply will only run once per instance - once we have rows it will
        not rerun, even if the arguments change!

        Arguments:
            args, kwargs: Arguments to the plugin.
        """
        if self._applied_args is None:
            self._applied_args = (args, kwargs)
        else:
            if self._applied_args != (args, kwargs):
                raise ValueError("%r was previously called with %r but is now "
                        "being called with %r." % (
                            self, self._applied_args, (args, kwargs)))
            else:
                # Results already cached.
                return self

        if issubclass(self.plugin_cls, plugin.TypedProfileCommand):
            # We have access to table header and rows without running render.
            self.table_header = self.plugin_cls.table_header
            plugin_curry = getattr(self.session.plugins, self.plugin_cls.name)
            command = plugin_curry(session=self.session, *args, **kwargs)
            self.rows = list(command.collect_as_dicts())
            self.columns = self.table_header.header
        else:
            # We do not have a table header declaration, so we need to run
            # the plugin and use an identity renderer to capture its output
            # and headers.

            # The identity renderer will capture rendered rows.
            renderer = identity_renderer.IdentityRenderer()

            self.session.RunPlugin(self.plugin_cls.name, format=renderer,
                                   *args, **kwargs)

            self.rows = renderer.rows
            self.columns = renderer.columns

        return self

    # IRepeated

    def getvalues(self):
        """Pretend the plugin is an IRepeated instead of a function."""
        # If we're being used as a repeated value (not a function) then we could
        # get here without apply already having been called.
        if self._applied_args is None:
            self.apply((), {})

        # Just return self, which is iterable.
        return self

    def value_eq(self, other):
        """This is required by the IRepeated protocol, but not actually used."""
        return self.getvalues() == repeated.getvalues(other)

    def value_apply(self, f):
        """This is required by the IRepeated protocol, but not actually used."""
        return repeated.meld(*[f(r) for r in self.rows])

    def value_type(self):
        """This is required by the IRepeated protocol, but not actually used."""
        return self.plugin_cls


# Implement the relevant EFILTER protocols using methods already on the
# CommandWrapper class.
repeated.IRepeated.implicit_static(CommandWrapper)
applicative.IApplicative.implicit_static(CommandWrapper)


class EfilterPlugin(plugin.ProfileCommand):
    """Abstract base class for plugins that do something with queries.

    Provides implementations of the basic EFILTER protocols for selecting and
    inspecting the output of plugins. Search and Explain extend this.
    """
    __abstract = True

    query = None  # The Query instance we're working with.
    query_source = None  # The source of the query, passed by the user.
    query_error = None  # An exception, if any, caused when parsing the query.

    # We cache CommandWrapper instances because they end up containing header
    # information for untyped plugins.
    _cached_command_wrappers = None

    # Plugin lifecycle:

    @classmethod
    def args(cls, parser):
        super(EfilterPlugin, cls).args(parser)
        parser.add_argument(
            "query",
            required=True,
            help="The dotty/EFILTER query to run.")

        parser.add_argument(
            "query_parameters",
            default=[],
            type="ArrayStringParser",
            help="Positional parameters for parametrized queries.")

    def __init__(self, query=None, query_parameters=None, **kwargs):
        super(EfilterPlugin, self).__init__(**kwargs)

        if not query:
            raise ValueError("You must supply a query (got %r)." % (query,))

        self.query_source = query

        try:
            self.query = q.Query(query, params=query_parameters)
        except errors.EfilterError as error:
            self.query_error = error
            self.query = None
        except Exception:
            # I am using a broad except here to make sure we always display a
            # friendly error message. EFILTER will usually raise a friendly
            # error, but we might get a non-EfilterError exception if the user
            # gets creative (e.g. passing a custom object as query, instead of a
            # string).
            raise ValueError("Could not parse your query %r." % (query,))

        self.query_parameters = query_parameters
        self._cached_command_wrappers = dict()

    # IStructured implementation for EFILTER:

    def resolve(self, name):
        """Find and return a CommandWrapper for the plugin 'name'."""
        if name in self.getmembers_runtime():
            return self._build_plugin_wrapper(name)

        raise KeyError("No plugin named %r." % name)

    def _build_plugin_wrapper(self, name):
        """Get a wrapper around the plugin called 'name'.

        Arguments:
            name: Plugin name to find.

        Returns:
            Instance of CommandWrapper.
        """
        meta = self.session.plugins.plugin_db.GetActivePlugin(name)
        wrapper = CommandWrapper(meta.plugin_cls, self.session)

        # We build the cache but don't retrieve wrappers from it. We need to
        # build a new wrapper every time a plugin is resolved from the query
        # because plugins are functions and might be called with different args
        # every time.
        self._cached_command_wrappers[name] = wrapper

        return wrapper

    def getmembers_runtime(self):
        """Get all available plugins."""
        return frozenset(
            [c.name for c in plugin.Command.GetActiveClasses(self.session)])

    def reflect_runtime_member(self, name):
        """Find the type* of 'name', which is a plugin.

        * This returns the plugin instance, not its class, because the entire
        reflection API requires information only available to Rekall at runtime.
        """
        cls = self.session.plugins.plugin_db.GetActivePlugin(name).plugin_cls

        # Does this plugin implement the reflection helper?
        try:
            return cls.GetPrototype(session=self.session)
        except NotImplementedError:
            # GetPrototype is not overriden and the default implementation
            # didn't work.
            return None

    # Plugin methods:

    def render_error(self, renderer):
        """Render the query parsing error in a user-friendly manner."""
        renderer.section("Query Error")

        try:
            start = self.query_error.adjusted_start
            end = self.query_error.adjusted_end
            source = self.query_error.source
            text = self.query_error.text
        except AttributeError:
            # Maybe query_error isn't a subclass of EfilterError. Let's be
            # careful.
            start = None
            end = None
            source = self.query_source
            text = str(self.query_error)

        if start is not None and end is not None:
            renderer.format(
                "EFILTER error ({}) {} at position {}-{} in query:\n{}\n\n",
                type(self.query_error).__name__, repr(text), start, end,
                utils.AttributedString(
                    source,
                    [dict(start=start, end=end, fg="RED", bold=True)]))
        else:
            renderer.format(
                "EFILTER error ({}) {} in query:\n{}\n",
                type(self.query_error).__name__, repr(text), source)

    def render(self, renderer):
        raise NotImplementedError()


structured.IStructured.implicit_dynamic(EfilterPlugin)


class Search(EfilterPlugin):
    """Searches and recombines output of other plugins.

    Search allows you to use the EFILTER search engine to filter, transform
    and combine output of most Rekall plugins. The most common use for this
    is running IOCs.

    Some examples that work right now:
    ==================================

    # Find the process with pid 1:
    search("select * pslist where proc.pid == 1")

    # Sort lsof output by file descriptor:
    search("sort(lsof, fd)") # or:
    search("select * from lsof order by fd)")

    # Filter and sort through lsof in one step:
    search("select * from lsof where proc.pid == 1 order by fd)

    # Is there any proc with PID 1, that has a TCPv6 connection and isn't a
    # dead process?
    search("(any lsof where (proc.pid == 1 and fileproc.human_type == 'TCPv6'))
             and not (any dead_procs where (proc.pid == 1))")

    # Note: "ANY" is just a short hand for "SELECT ANY FROM" which does what
    # it sounds like, and returns True or False depending on whether the
    # query has any results.
    """

    name = "search"

    @classmethod
    def args(cls, parser):
        super(Search, cls).args(parser)
        parser.add_argument(
            "silent",
            default=False,
            help="Queries should fail silently.")

    def __init__(self, *args, **kwargs):
        self.silent = kwargs.pop("silent", None)
        super(Search, self).__init__(*args, **kwargs)

    def collect(self):
        """Return the search results without displaying them.

        Returns:
            A list of results from the query solver.

        Raises:
            EfilterError unless 'silent' flag was set.
        """
        try:
            result = self.solve()
            return repeated.getvalues(result)
        except errors.EfilterError:
            if self.silent:
                return None

            raise

    def solve(self):
        """Return the search results exactly as EFILTER returns them.

        Returns:
            Depends on the query.

        Raises:
            EfilterError if anything goes wrong.
        """
        return solve.solve(self.query, self).value

    @utils.safe_property
    def first_result(self):
        """Get only the first search result.

        This is useful when we need to find a concrete structure for some other
        purpose, such as finding a concrete allocator zone when writing a
        'dump_zone' plugin.
        """
        try:
            for result in self.collect():
                return result
        except (TypeError, ValueError):
            return None

    def _render_plugin_output(self, renderer, table_header, *rows):
        """Used to render search results if they come from a plugin."""
        try:
            columns = [column["cname"] for column in table_header]
        except KeyError:
            raise ValueError(
                "Column spec %r is missing a cname. Full header was: %r." %
                (column, table_header))

        for row in rows:
            renderer.table_row(*[row[key] for key in columns])

    def _render_dicts(self, renderer, *rows):
        """Used to render search results if they are basic dicts."""
        for row in rows:
            renderer.table_row(*row.itervalues())

    def _render_whatever_i_guess(self, renderer, *rows):
        """Used to render search results if we don't know WTF they are."""
        for row in rows:
            renderer.table_row(row)

    def _find_matching_header(self, keys):
        sorted_keys = sorted(keys)
        for cached_wrapper in self._cached_command_wrappers.itervalues():
            if not cached_wrapper.columns:
                continue

            column_names = [c.get("cname", c.get("name"))
                            for c in cached_wrapper.columns]
            if sorted(column_names) == sorted_keys:
                return cached_wrapper.columns


    def render(self, renderer):
        # Do we have a query?
        if not self.query:
            return self.render_error(renderer)

        # Figure out what the header should look like.
        # Can we infer the type?
        try:
            t = infer_type.infer_type(self.query, self)
        except Exception:
            t = None

        # Get the data we're rendering.
        try:
            rows = self.collect() or []
        except errors.EfilterError as error:
            self.query_error = error
            return self.render_error(renderer)

        # If the output type is a TypeProfileCommand subclass then we can just
        # interrogate its header.
        if isinstance(t, plugin.TypedProfileCommand):
            renderer.table_header(t.table_header)
            return self._render_plugin_output(renderer, t.table_header,
                                              *rows)

        # If the output type is a regular plugin then we must've run it at some
        # point and should be able to retrieve a cached copy of the wrapper,
        # which will have preserved the output columns.
        if isinstance(t, plugin.Command):
            cached_wrapper = self._cached_command_wrappers.get(t.name)
            if not cached_wrapper:
                raise RuntimeError("Command of type %r is the output of an "
                                   "EFILTER query but no such command was "
                                   "executed." % (t,))

            if not cached_wrapper._applied_args:
                cached_wrapper.apply((), {})

            renderer.table_header(cached_wrapper.columns)
            return self._render_plugin_output(renderer, cached_wrapper.columns,
                                              *rows)

        # If we got no rows in the output the just say so.
        rows = iter(rows)
        try:
            first_row = next(rows)
        except StopIteration:
            renderer.table_header([("No Results", "no_results", "20")])
            return

        # As last ditch, try to guess the header based on the data in the
        # first row.
        if isinstance(first_row, dict):
            # Maybe we have a plugin with matching columns in its output?
            columns = self._find_matching_header(first_row.keys())
            if columns:
                renderer.table_header(columns)
                return self._render_plugin_output(renderer,
                                                  columns,
                                                  first_row,
                                                  *rows)

            renderer.table_header(
                [dict(name=unicode(k), cname=unicode(k))
                 for k in first_row.iterkeys()])

            return self._render_dicts(renderer, first_row, *rows)

        # Sigh. Give up, and render whatever you got, I guess.
        renderer.table_header([dict(name="Result", cname="result")])
        return self._render_whatever_i_guess(renderer, first_row, *rows)


class Explain(EfilterPlugin):
    """Prints various information about a query.

    Explains how a query was parsed and how it will be interpreted. It also
    runs a full type inferencer, to attempt to determine the output of the
    query once it's executed.

    The Explain plugin can analyse a strict superset of expressions that
    are valid in the Search plugin. It supports:

     - Any search query that can be passed to Search.
     - Expressions asking about types and members of profile types
       (like structs).
    """

    name = "explain"

    # As long as this is True, the input is a valid search query and will be
    # analysed in the output. This may become False if we realize the input
    # is not a valid search query, but instead asking about something like the
    # structure of a native type.
    input_is_regular_query = True

    def reflect_runtime_member(self, name):
        """Reflect what Search reflects, and also struct types."""
        result = super(Explain, self).reflect_runtime_member(name)

        if not result or result == protocol.AnyType:
            result = self.session.profile.GetPrototype(name)
            if result and result != protocol.AnyType:
                # We found something that makes this not a query (aka a struct).
                self.input_is_regular_query = False

        return result

    def getmembers_runtime(self):
        """Reflect what Search reflects, and also struct types."""
        result = super(Explain, self).getmembers_runtime()

        return set(result) | set(self.session.profile.vtypes.iterkeys())

    def recurse_expr(self, expr, depth):
        yield expr, depth

        if not isinstance(expr, ast.Expression):
            return

        for child in expr.children:
            for expr_, depth in self.recurse_expr(child, depth + 1):
                yield expr_, depth

    def _render_node(self, query, node, renderer, depth=1):
        """Render an AST node and recurse."""
        t = infer_type.infer_type(node, self)

        try:
            name = "(%s) <%s>" % (t.__name__, type(node).__name__)
        except AttributeError:
            name = "(%r) <%s>" % (t, type(node).__name__)

        renderer.table_row(
            name,
            utils.AttributedString(
                str(query),
                [dict(start=node.start, end=node.end, fg="RED", bold=True)]
            ),
            depth=depth
        )

        for child in node.children:
            if isinstance(child, ast.Expression):
                self._render_node(node=child, renderer=renderer, query=query,
                                  depth=depth + 1)
            else:
                renderer.table_row(
                    "(%s) <leaf: %r>" % (type(child).__name__, child),
                    None,
                    depth=depth + 1
                )

    def render(self, renderer):
        # Do we have a query?
        if not self.query:
            return self.render_error(renderer)

        # render_output_analysis must run before render_query_analysis
        # because it decides whether the input is a regular query.
        self.render_output_analysis(renderer)
        self.render_query_analysis(renderer)

    def render_output_analysis(self, renderer):
        """Render analysis of the expression's return type and its members."""
        output_type = infer_type.infer_type(self.query, self)

        renderer.section("Type Analysis", width=140)
        renderer.table_header([
            dict(name="Name", cname="name", type="TreeNode", max_depth=2,
                 width=60),
            dict(name="Type", cname="type", width=40)
        ])

        renderer.table_row(self.query.source,
                           repr(output_type),
                           depth=1)

        try:
            for member in structured.getmembers(output_type):
                subq = "(%s)[%r]" % (self.query.source, member)
                subtype = infer_type.infer_type(q.Query(subq), self)
                if isinstance(subtype, type):
                    subtype = subtype.__name__
                else:
                    subtype = repr(subtype)

                renderer.table_row(subq, subtype, depth=2)
        except (NotImplementedError, TypeError):
            pass

    def render_query_analysis(self, renderer):
        """Render query analysis if the input is a regular query.

        A non-regular query could be the user asking us to explain (e.g.) a
        struct.
        """
        if not self.input_is_regular_query:
            return

        original_query = self.query.source
        canonical_query = asdottysql.asdottysql(self.query)

        renderer.section("Query Analysis", width=140)
        self.render_query(renderer, self.query)

        if canonical_query != original_query:
            renderer.section("Query Analysis (Using canonical syntax)",
                             width=140)
            self.render_query(renderer, q.Query(canonical_query))

    def render_query(self, renderer, query):
        """Render a single query object's analysis."""
        renderer.table_header([
            dict(name="(Return Type) Expression", cname="expression",
                 type="TreeNode", max_depth=15, width=40),
            dict(name="Subquery", cname="query", width=100, nowrap=True),
        ])

        self._render_node(query, query.root, renderer)


# Below we implement various EFILTER protocols for various Rekall types.


# Implement IApplicative for Command to get reflection APIs.
applicative.IApplicative.implement(
    for_type=plugin.Command,
    implementations={
        applicative.apply:
            lambda x, *args, **kwargs: x(*args, **kwargs).collect(),

        # Plugins "return" themselves, as far as the type inference cares.
        applicative.reflect_runtime_return: lambda command: command
    }
)


# TypedProfileCommands can reflect a lot about their output columns.
# The 'resolve' function will never actually be called on TypeProfileCommand,
# because we treat plugins as tables, not rows. 'resolve' will instead be
# passed the rowdicts.
structured.IStructured.implement(
    for_type=plugin.TypedProfileCommand,
    implementations={
        structured.resolve: lambda _, __: None,  # This should not happen.
        structured.reflect_runtime_member:
        lambda c, name: c.get_column_type(name),
        structured.getmembers_runtime: lambda c: c.table_header.all_names
    }
)


# We support IAssociative (plugin[column]) using the same accessors as
# IStructured (plugin.column). We're easy-going like that.
# As with IStructured, the 'select' function doesn't get called on the
# plugin itself, which is why we don't provide a real implementation.
associative.IAssociative.implement(
    for_type=plugin.TypedProfileCommand,
    implementations={
        associative.select: lambda _, __: None,  # This should not happen.
        associative.reflect_runtime_key:
        lambda c, name: c.get_column_type(name),
        associative.getkeys_runtime: lambda c: c.table_header.all_names
    }
)


# Implement IAssociative for Structs because why not. This lets us do
# struct[key] as well as struct.key.
associative.IAssociative.implement(
    for_type=obj.Struct,
    implementations={
        associative.select: getattr,
        associative.reflect_runtime_key: structured.reflect_runtime_member,
        associative.getkeys_runtime: structured.getmembers_runtime
    }
)


# This lets us do struct.member.
structured.IStructured.implement(
    for_type=obj.Struct,
    implementations={
        structured.resolve: getattr,
        structured.reflect_runtime_member:
            lambda s, m: type(getattr(s, m, None)),
        structured.getmembers_runtime:
            lambda s: set((name for name, _ in s.getproperties()))
    }
)


# This lets us get indices out of Arrays.
associative.IAssociative.implement(
    for_type=obj.Array,
    implementations={
        associative.select: lambda obj, key: obj[key],
    }
)


# This lets us do some_array.some_member. Useful for accessing properties.
structured.IStructured.implement(
    for_type=obj.Array,
    implementations={
        structured.resolve: getattr
    }
)


def select_Pointer(ptr, key):
    """Delegate to target of the pointer, if any."""
    obj = ptr.deref()
    if not obj:
        ptr.session.logging.warn(
            "Attempting to access key %r of a void pointer %r.", key, ptr)
    if obj:
        return associative.select(obj, key)


# Pointer[key] is implemented as Pointer.dereference()[key].
associative.IAssociative.implement(
    for_type=obj.Pointer,
    implementations={
        associative.select: select_Pointer
    }
)


def resolve_Pointer(ptr, member):
    """Delegate to target of the pointer, if any."""
    obj = ptr.deref()
    if not obj:
        ptr.session.logging.warn(
            "Attempting to access member %r of a void pointer %r.", member, ptr)
    if obj:
        return structured.resolve(obj, member)


# Pointer.member is implemented as Pointer.dereference().member.
structured.IStructured.implement(
    for_type=obj.Pointer,
    implementations={
        structured.resolve: resolve_Pointer
    }
)



#!/usr/bin/env python

# Rekall
# Copyright (C) 2012 Michael Cohen <scudette@gmail.com>
# Copyright 2013 Google Inc. All Rights Reserved.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or (at
# your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
#

__author__ = "Michael Cohen <scudette@gmail.com>"

# pylint: disable=protected-access

import logging
import pdb
import sys

import rekall
from rekall import args
from rekall import config
from rekall import constants
from rekall import plugin
from rekall import session

from pkg_resources import iter_entry_points
for entry_point in iter_entry_points(group='rekall.plugins', name=None):
    entry_point.load()

# Load all the plugins.
from rekall import plugins  # pylint: disable=unused-import


config.DeclareOption(
    "--live", default=False, type="Boolean",
    help="Enable live memory analysis.")

config.DeclareOption(
    "--version", default=False, type="Boolean",
    help="Prints the Rekall version and exits.")


class Run(plugin.PrivilegedMixIn, plugin.Command):
    """A plugin which runs its argument (using eval).

    Note: This plugin is only defined and available when using the main entry
    point. It is not available when Rekall is used as a library since it allows
    arbitrary code execution.
    """

    name = "run"

    @classmethod
    def args(cls, parser):
        super(Run, cls).args(parser)
        parser.add_argument("script", default="print 'hello!'",
                            help="The script to evaluate")

        parser.add_argument("--run", default=None,
                            help="A file name to run.")

    def __init__(self, script, run=None, **kwargs):
        super(Run, self).__init__(**kwargs)
        if run is not None:
            script = open(run).read()

        exec script in self.session.locals


def main(argv=None):
    # New user interactive session (with extra bells and whistles).
    user_session = session.InteractiveSession()
    user_session.session_list.append(user_session)

    # Alow all special plugins to run.
    user_session.privileged = True

    def global_arg_cb(global_flags, _):
        if global_flags.version:
            print "This is Rekall Version %s (%s)" % (
                constants.VERSION, constants.CODENAME)

            print rekall.get_versions()
            sys.exit(0)

    plugin_cls, flags = args.parse_args(
        argv=argv, global_arg_cb=global_arg_cb,
        user_session=user_session)

    try:
        # Run the plugin with plugin specific args.
        user_session.RunPlugin(plugin_cls, **config.RemoveGlobalOptions(flags))
    except Exception as e:
        logging.fatal("%s. Try --debug for more information." % e)
        if getattr(flags, "debug", None):
            pdb.post_mortem(sys.exc_info()[2])
        raise
    finally:
        user_session.Flush()

if __name__ == '__main__':
    main()



#!/usr/bin/env python2

# Rekall Memory Forensics
# Copyright 2015 Google Inc. All Rights Reserved.
#
# Author: Michael Cohen scudette@google.com
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or (at
# your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
#

__author__ = "Michael Cohen <scudette@google.com>"

"""A plugin to install relevant kernel modules to enable live analysis.

The intention is to allow the user to launch:

rekall live

and have Rekall install the right kernel module and connect to the driver on all
supported operating systems.
"""
from rekall import plugin

from rekall.plugins.addrspaces import elfcore
from rekall.plugins.addrspaces import standard


class Live(plugin.ProfileCommand):
    """Launch a Rekall shell for live analysis on the current system."""

    name = "live"

    PROFILE_REQUIRED = False

    def live(self):
        try:
            base_as = standard.FileAddressSpace(session=self.session,
                                                filename="/proc/kcore")
            self.session.physical_address_space = elfcore.KCoreAddressSpace(
                base=base_as)
        except IOError as e:
            self.session.logging.debug("%s", e)
            raise plugin.PluginError("%s. Are you root?" % e)

    def close(self):
        pass

    def __str__(self):
        # The default __str__ form will run the plugin which will drop into a
        # shell!
        return "Live Plugin"

    def render(self, renderer):
        renderer.format("Launching live memory analysis\n")
        self.live()

        # Launch the shell.
        shell = self.session.plugins.shell()
        shell.render(renderer)



# Rekall Memory Forensics
# Copyright (C) 2012 Michael Cohen
# Copyright 2014 Google Inc. All Rights Reserved.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or (at
# your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
#

"""This module implements a pass-through renderer.

The renderer in this module just passes table rows along as it received them.
"""

from rekall.ui import renderer as renderer_module


class IdentityRenderer(renderer_module.BaseRenderer):
    columns = None
    plugin_name = None
    rows = None

    def __init__(self, *_, **__):
        super(IdentityRenderer, self).__init__()
        self.rows = []

    def __enter__(self):
        return self

    def __exit__(self, *_):
        pass

    def start(self, plugin_name=None, **_):
        self.plugin_name = plugin_name
        return self

    def table_header(self, columns=None, **_):
        self.columns = []
        for column in columns:
            if isinstance(column, dict):
                self.columns.append(column)
            elif isinstance(column, tuple):
                self.columns.append(dict(name=column[0],
                                         cname=column[1],
                                         formatstring=column[2]))
            else:
                raise TypeError("Column spec must be dict or tuple. Got %r."
                                % column)

    def _get_column_name(self, idx):
        column = self.columns[idx]
        if isinstance(column, tuple):
            return column[1]

        return column.get("cname", column.get("name"))

    def table_row(self, *values, **_):
        row = dict()
        for idx, value in enumerate(values):
            cname = self._get_column_name(idx)
            row[cname] = value

        self.rows.append(row)



# Rekall Memory Forensics
# Copyright (C) 2012 Michael Cohen <scudette@users.sourceforge.net>
# Copyright (c) 2008 Volatile Systems
# Copyright (c) 2008 Brendan Dolan-Gavitt <bdolangavitt@wesleyan.edu>
# Copyright 2013 Google Inc. All Rights Reserved.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or (at
# your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
#

# pylint: disable=protected-access

from rekall import obj

from rekall.plugins.overlays.windows import common
from rekall.plugins.overlays.windows import heap
from rekall.plugins.overlays.windows import pe_vtypes
from rekall.plugins.overlays.windows import tokens

from rekall.plugins.overlays.windows import xp
from rekall.plugins.overlays.windows import vista
from rekall.plugins.overlays.windows import win7
from rekall.plugins.overlays.windows import win8
from rekall.plugins.overlays.windows import win10
from rekall.plugins.overlays.windows import crashdump
from rekall.plugins.overlays.windows import undocumented



class Ntoskrnl(pe_vtypes.BasicPEProfile):
    """A profile for Windows."""

    @classmethod
    def GuessVersion(cls, profile):
        """Guess the windows version of a profile."""
        # If the version is provided, then just use it.
        try:
            major, minor = profile.metadatas("major", "minor")
            version = major + minor / 10.0
            profile.set_metadata("version", version)

            return version
        except TypeError:
            pass

        # Rekall is moving away from having features keyed by version, rather we
        # use the profile to dictate the algorithms to use. In future we will
        # remove all requirement to know the windows version, but for now we
        # just guess the version based on structures which are known to exist in
        # the profile.
        version = 5.2

        # Windows XP did not use a BalancedRoot for VADs.
        if profile.get_obj_offset("_MM_AVL_TABLE", "BalancedRoot") == None:
            version = 5.1

        # Windows 7 introduces TypeIndex into the object header.
        if profile.get_obj_offset("_OBJECT_HEADER", "TypeIndex") != None:
            # Windows 10 introduces a cookie for object types.
            if profile.get_constant("ObHeaderCookie"):
                version = 10.0

            elif profile._EPROCESS().m(
                    "VadRoot.BalancedRoot").obj_type == "_MMADDRESS_NODE":
                version = 6.1

            elif profile._EPROCESS().m("VadRoot").obj_type == "_MM_AVL_TABLE":
                # Windows 8 uses _MM_AVL_NODE as the VAD traversor struct.
                version = 6.2

            elif profile._EPROCESS().m("VadRoot").obj_type == "_RTL_AVL_TREE":
                # Windows 8.1 and on uses _RTL_AVL_TREE
                version = 6.3

            else:
                raise RuntimeError("Unknown windows version")

        profile.set_metadata("version", version)
        major, minor = divmod(version, 1)
        profile.set_metadata("minor", int(minor * 10))
        profile.set_metadata("major", major)

        return version

    @classmethod
    def Initialize(cls, profile):
        super(Ntoskrnl, cls).Initialize(profile)

        profile.add_enums(**undocumented.ENUMS)
        if profile.metadata("arch") == "AMD64":
            profile.add_overlay(undocumented.AMD64)

        elif profile.metadata("arch") == "I386":
            profile.add_overlay(undocumented.I386)

            # Detect if this is a PAE system. PAE systems have 64 bit PTEs:
            if profile.get_obj_size("_MMPTE") == 8:
                profile.set_metadata("pae", True)

        # Install the base windows support.
        common.InitializeWindowsProfile(profile)
        crashdump.InstallKDDebuggerProfile(profile)
        tokens.InitializeTokenProfiles(profile)
        heap.InitializeHeapProfile(profile)

        # Get the windows version of this profile.
        version = cls.GuessVersion(profile)
        if 10 <= version:
            win10.InitializeWindows10Profile(profile)

        elif 6.2 <= version < 10:
            win8.InitializeWindows8Profile(profile)

        elif version == 6.1:
            win7.InitializeWindows7Profile(profile)

        elif version == 6.0:
            vista.InitializeVistaProfile(profile)

        elif 5.1 <= version <= 5.2:
            xp.InitializeXPProfile(profile)

    def GetImageBase(self):
        if not self.image_base:
            self.image_base = obj.Pointer.integer_to_address(
                self.session.GetParameter("kernel_base"))

        return self.image_base


class Ntkrnlmp(Ntoskrnl):
    """Alias for the windows kernel class."""


class Ntkrnlpa(Ntoskrnl):
    """Alias for the windows kernel class."""


class Ntkrpamp(Ntoskrnl):
    """Alias for the windows kernel class."""


class Nt(Ntoskrnl):
    """Alias for the windows kernel class."""



#!/usr/bin/env python

# Rekall
# Copyright 2013 Google Inc. All Rights Reserved.
#
# Authors:
# Michael Cohen <scudette@gmail.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or (at
# your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
#
"""Installation and deployment script."""
__author__ = "Michael Cohen <scudette@gmail.com>"
import platform
import os
import subprocess
import versioneer

from setuptools import find_packages, setup, Command

rekall_description = "Rekall Memory Forensic Framework"

current_directory = os.path.dirname(__file__)


def find_data_files(source):
    result = []
    for directory, _, files in os.walk(source):
        files = [os.path.join(directory, x) for x in files]
        result.append((directory, files))

    return result

# These versions are fixed to the exact tested configuration. Prior to release,
# please use "setup.py pip_upgrade" to test with the latest version. This
# approach ensures that any Rekall version will always work as tested - even
# when external packages are upgraded in an incompatible way.
install_requires = [
    "PyAFF4 == 0.23",
    "PyYAML == 3.11",
    "acora == 1.9",
    "argparse == 1.2.1",
    "arrow == 0.7.0",
    "efilter == 1453815385",
    "intervaltree == 2.1.0",
    "pycrypto == 2.6.1",
    "pyelftools == 0.23",
    "pytz == 2015.7",
    "rekall-capstone == 3.0.4.post2",
    "rekall-yara == 3.4.0.1",
    "sortedcontainers == 1.4.4",

    # Version 2.5.0 is broken with pyinstaller.
    # https://github.com/pyinstaller/pyinstaller/issues/1848
    "python-dateutil == 2.4.2",
]

if platform.system() == "Windows":
    install_requires.append("pypiwin32 == 219")


if "VIRTUAL_ENV" not in os.environ:
    print "*****************************************************"
    print "  WARNING: You are not installing Rekall in a virtual"
    print "  environment. This configuration is not supported!!!"
    print "  Expect breakage."
    print "*****************************************************"


class PIPUpgrade(Command):
    description = "Upgrade all the dependencies in the current virtualenv."
    user_options = []
    def initialize_options(self):
        pass

    def finalize_options(self):
        pass

    def run(self):
        required = [x.split()[0] for x in install_requires]
        output = subprocess.check_output(
            ["pip", "install", "--upgrade"] + required)

        # Print the current versions.
        output = subprocess.check_output(
            ["pip", "freeze"])

        result = []
        for package in required:
            result.append(
                [x for x in output.splitlines()
                 if package in x][0])

        print "\n".join(sorted(result))


class CleanCommand(Command):
    description = ("custom clean command that forcefully removes "
                   "dist/build directories")
    user_options = []
    def initialize_options(self):
        self.cwd = None
    def finalize_options(self):
        self.cwd = os.getcwd()
    def run(self):
        if os.getcwd() != self.cwd:
            raise RuntimeError('Must be in package root: %s' % self.cwd)

        os.system('rm -rf ./build ./dist')

commands = versioneer.get_cmdclass()
commands["pip_upgrade"] = PIPUpgrade
commands["clean"] = CleanCommand


setup(
    name="rekall-core",
    version=versioneer.get_version(),
    cmdclass=commands,
    description=rekall_description,
    long_description=open(os.path.join(current_directory, "README.rst")).read(),
    license="GPL",
    url="https://www.rekall-forensic.com/",
    author="The Rekall team",
    author_email="rekall-discuss@googlegroups.com",
    classifiers=[
        "Development Status :: 4 - Beta",
        "Environment :: Console",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
    ],
    scripts=["rekall/rekal.py"],
    package_dir={'rekall': 'rekall'},
    packages=find_packages('.'),
    data_files=find_data_files("resources"),

    entry_points="""
    [rekall.plugins]
    plugins=rekall.plugins

    [console_scripts]
    rekal = rekall.rekal:main
    rekall = rekall.rekal:main
    """,
    install_requires=install_requires,
)



# Rekall Memory Forensics
# Copyright 2015 Google Inc. All Rights Reserved.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or (at
# your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
#

"""This module implements renderers specific to darwin structures."""

from rekall.ui import json_renderer

from rekall.ui import text
from rekall.plugins.renderers import base_objects
from rekall.plugins.renderers import data_export


class ProcDataExport(data_export.DataExportBaseObjectRenderer):
    renders_type = "proc"

    def EncodeToJsonSafe(self, task, **_):
        result = super(ProcDataExport, self).EncodeToJsonSafe(task)
        result["Cybox"] = dict(
            type=u"ProcessObj:ProcessObjectType",
            Name=task.name,
            PID=task.pid,
            Creation_Time=task.p_start,
            Parent_PID=task.p_ppid,
            Image_Info=dict(
                type=u"ProcessObj:ImageInfoType",
                Path=task.p_comm,
                Command_Line=task.p_comm,
                File_Name=task.p_comm))

        res = json_renderer.JsonObjectRenderer.EncodeToJsonSafe(self, result)
        return res

    def Summary(self, item, **_):
        return "%s (%s)" % (item.get("Cybox", {}).get("Name", ""),
                            item.get("Cybox", {}).get("PID", ""))


class Fileproc_TextObjectRenderer(base_objects.StructTextRenderer):
    renders_type = "fileproc"

    COLUMNS = [
        dict(name="Type", cname="human_type", width=15),
        dict(name="Name", cname="human_name", width=40)
    ]


class Vnode_TextObjectRenderer(base_objects.StructTextRenderer):
    renders_type = "vnode"

    COLUMNS = [
        dict(name="Vnode", cname="obj_offset", style="address"),
        dict(name="Path", cname="full_path", width=40, nowrap=True)
    ]


class Clist_TextObjectRenderer(base_objects.StructTextRenderer):
    renders_type = "clist"

    COLUMNS = [
        dict(name="Clist", cname="obj_offset", style="address"),
        dict(name="Recovered Contents", cname="recovered_contents",
             width=34, style="hexdump")
    ]


class Tty_TextObjectRenderer(base_objects.StructTextRenderer):
    renders_type = "tty"

    COLUMNS = [
        dict(name="TTY", style="address", cname="obj_offset"),
        dict(name="Vnode", type="vnode", cname="vnode"),
        dict(name="Input Buffer", type="clist", cname="input_buffer",
             columns=[dict(name="Recovered Input",
                           cname="recovered_contents",
                           style="hexdump",
                           width=34)]),
        dict(name="Output Buffer", type="clist", cname="output_buffer",
             columns=[dict(name="Recovered Output",
                           cname="recovered_contents",
                           style="hexdump",
                           width=34)])
    ]


class Session_TextObjectRenderer(base_objects.StructTextRenderer):
    renders_type = "session"

    COLUMNS = [
        dict(name="Session", cname="obj_offset", style="address"),
        dict(name="Session ID", cname="s_sid"),
        dict(name="Leader", cname="s_leader", type="proc",
             columns=[dict(name="Leader PID", cname="pid"),
                      dict(name="Leader Command", cname="command",
                           width=30)]),
        dict(name="Login", cname="s_login", width=20, nowrap=True)
    ]


class Socket_TextObjectRenderer(base_objects.StructTextRenderer):
    renders_type = "socket"

    COLUMNS = [
        dict(name="Socket", cname="obj_offset", style="address"),
        dict(name="Last PID", cname="last_pid", width=10),
        dict(name="Type", cname="human_type", width=20),
        dict(name="Description", cname="human_name", width=60)
    ]


class Rtentry_TextObjectRenderer(base_objects.StructTextRenderer):
    renders_type = "rtentry"

    COLUMNS = [
        dict(name="IP Address", cname="source_ip", type="sockaddr",
             width=18),
        dict(name="Mac Address", cname="dest_ip", type="sockaddr",
             width=18),
        dict(name="Interface", cname="name", align="c"),
        dict(name="Sent", cname="sent", width=8, align="r"),
        dict(name="Received", cname="rx", width=8, align="r"),
        dict(name="Time", cname="base_calendartime", width=30, align="c"),
        dict(name="Expires", cname="rt_expire", align="r"),
        dict(name="Delta", cname="delta", align="r")
    ]


class Sockaddr_TextObjectRenderer(text.TextObjectRenderer):
    renders_type = "sockaddr"

    def render_full(self, target, **_):
        return text.Cell(target.address)


class Zone_TextObjectRenderer(base_objects.StructTextRenderer):
    renders_type = "zone"
    COLUMNS = [
        dict(name="Name", cname="name", width=20),
        dict(name="Active Count", cname="count_active", width=12),
        dict(name="Free Count", cname="count_free", width=12),
        dict(name="Element Size", cname="elem_size", width=12),
        dict(name="Tracks Pages", cname="tracks_pages", width=12),
        dict(name="Allows Foreign", cname="allows_foreign", width=12)
    ]


class Ifnet_TextObjectRenderer(base_objects.StructTextRenderer):
    renders_type = "ifnet"
    COLUMNS = [
        dict(name="Interface", cname="name", width=12),
        dict(name="MAC", cname="l2_addr", width=18),
        dict(name="IPv4", cname="ipv4_addr", width=16),
        dict(name="IPv6", cname="ipv6_addr", width=40)
    ]


class Proc_TextObjectRenderer(base_objects.StructTextRenderer):
    renders_type = "proc"
    COLUMNS = [
        dict(name="proc", style="address", cname="obj_offset"),
        dict(name="Name", width=20, align="l", cname="name"),
        dict(name="PID", width=5, align="r", cname="pid")
    ]



# Rekall Memory Forensics
# Copyright (C) 2012 Michael Cohen <scudette@gmail.com>
# Copyright 2013 Google Inc. All Rights Reserved.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or (at
# your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
#

"""
Provides the primitives needed to disassemble code using capstone.
"""

import binascii
import capstone
import re
import struct

from capstone import x86_const
from rekall import addrspace
from rekall import plugin
from rekall import obj
from rekall import utils
from rekall import testlib


class Disassembler(object):
    __abstract = True

    def __init__(self, mode, session=None, address_space=None):
        self.mode = mode
        self.session = session
        self.address_space = (
            address_space or
            addrspace.BaseAddressSpace.classes["DummyAddressSpace"](
                session=session))

    def disassemble(self, data, offset):
        """ Starts disassembly of data """

    def is_return(self):
        return False

    def is_branch(self):
        return False

    def target(self):
        return None


class Instruction(object):
    """A Decoded instruction."""
    __abstract = True


class CapstoneInstruction(Instruction):
    """A capstone decoded instruction."""

    # We need to build reverse maps to properly interpret capston
    # instructions.
    INSTRUCTIONS = {}
    REGISTERS = {}
    OP = {}

    @classmethod
    def _init_class(cls):
        for constant in dir(x86_const):
            components = constant.split("_")
            value = getattr(x86_const, constant)
            if components[0] == "X86":
                if components[1] == "INS":
                    cls.INSTRUCTIONS[value] = components[2]
                elif components[1] == "REG":
                    cls.REGISTERS[value] = components[2]
                elif components[1] == "OP":
                    cls.OP[value] = components[2]

        cls.REGISTERS[0] = None

    def __init__(self, insn, session=None, address_space=None):
        self.address_space = address_space
        self.insn = insn
        self.address = insn.address
        self.size = insn.size
        self.mnemonic = insn.mnemonic
        self._comment = ""
        self._operands = None  # Cache the operands.
        self.session = session
        self.resolver = session.address_resolver
        if not self.REGISTERS:
            self._init_class()

    @utils.safe_property
    def operands(self):
        if self._operands is not None:
            return self._operands

        result = []
        # For invalid instructions there are no operands
        if self.insn.id == 0:
            return result

        for op in self.insn.operands:
            operand = dict(type=self.OP[op.type], size=op.size)
            if operand["type"] == "REG":
                operand["reg"] = self.REGISTERS[op.reg]

            elif operand["type"] == "MEM":
                # This is of the form: [base_reg + disp + index_reg * scale]
                mem = op.mem
                operand["base"] = self.REGISTERS[mem.base]
                operand["disp"] = mem.disp
                operand["index"] = self.REGISTERS[mem.index]
                operand["scale"] = mem.scale

                if operand["base"] == "RIP":
                    target = self.insn.address + mem.disp + self.insn.size
                    operand["address"] = target
                    operand["target"] = self._read_target(target, operand)

                    self._comment = self.format_indirect(target, op.size)

                # Simple indirect address.
                if not operand["base"] and not operand["index"]:
                    operand["address"] = mem.disp
                    operand["target"] = self._read_target(mem.disp, operand)
                    self._comment = self.format_indirect(mem.disp, op.size)

            elif operand["type"] == "IMM":
                operand["target"] = operand["address"] = op.imm.real
                self._comment = ", ".join(self.resolver.format_address(
                    op.imm.real))

            result.append(operand)

        # Cache for next time.
        self._operands = result
        return result

    def _read_target(self, target, operand):
        data = self.address_space.read(target, operand["size"])
        if operand["size"] == 8:
            return struct.unpack("<Q", data)[0]

        if operand["size"] == 4:
            return struct.unpack("<I", data)[0]

    def GetCanonical(self):
        """Returns the canonical model of the instruction."""
        result = dict(mnemonic=self.INSTRUCTIONS[self.insn.id],
                      str="%s %s" % (self.insn.mnemonic, self.insn.op_str),
                      operands=self.operands)

        result["comment"] = self._comment
        return result

    @utils.safe_property
    def comment(self):
        return self.GetCanonical()["comment"]

    @utils.safe_property
    def op_str(self):
        return self.GetCanonical()["str"]

    @utils.safe_property
    def text(self):
        canonical = self.GetCanonical()
        if canonical["comment"]:
            return "%s (%s)" % (canonical["str"], canonical["comment"])
        return canonical["str"]

    @utils.safe_property
    def hexbytes(self):
        return unicode(binascii.hexlify(self.insn.bytes))

    def format_indirect(self, operand, size):
        if size == 1:
            type = "byte"
        elif size == 2:
            type = "unsigned short"
        elif size == 4:
            type = "unsigned int"
        else:
            type = "address"

        target = self.session.profile.Object(
            type, offset=operand, vm=self.address_space).v()

        target_name = ", ".join(self.resolver.format_address(target))
        operand_name = ", ".join(self.resolver.format_address(operand))

        if target_name:
            return "0x%x %s -> %s" % (target, operand_name, target_name)
        else:
            return "0x%x %s" % (target, operand_name)

    def is_return(self):
        return self.mnemonic.startswith("ret")

    # https://en.wikibooks.org/wiki/X86_Assembly/Control_Flow
    def is_branch(self):
        """Is this instruction a branch?

        e.g. JNE JE JG JLE JL JGE JMP JA JAE JB JBE JO JNO JZ JNZ JS JNS
        """
        return self.mnemonic.startswith("j")

    @utils.safe_property
    def target(self):
        if self.mnemonic[0] == "j":
            operand = self.operands[0]
            if operand["type"] in ("IMM", "MEM"):
                return operand.get("address")

            # We can not determine the target of REG jumps without the
            # registers.

    def match_rule(self, rule, context):
        """Match the rule against this instruction."""
        # Speed optimization. Most of the time the rule matches the mnemonic.
        mnemonic = rule.get("mnemonic")
        if mnemonic and mnemonic != self.INSTRUCTIONS[self.insn.id]:
            return False

        return self._MatchRule(rule, self.GetCanonical(), context)

    def _MatchRule(self, rule, instruction, context):
        if isinstance(rule, dict):
            for k, v in rule.iteritems():
                expected = instruction.get(k)
                if not self._MatchRule(v, expected, context):
                    return False
            return True

        if isinstance(rule, (list, tuple)):
            for subrule, subinst in zip(rule, instruction):
                if subrule and not self._MatchRule(subrule, subinst, context):
                    return False

            return True

        if isinstance(rule, basestring):
            # Rules starting with $ are capture variables.
            if rule[0] == "$":
                context[rule] = instruction
                return True

            # Rules starting with ~ are regular expressions.
            if isinstance(instruction, basestring) and rule[0] == "~":
                return re.match(rule[1:], instruction)

        return rule == instruction


class Capstone(Disassembler):
    def __init__(self, mode, **kwargs):
        super(Capstone, self).__init__(mode, **kwargs)

        if self.mode == "I386":
            self.cs = capstone.Cs(capstone.CS_ARCH_X86, capstone.CS_MODE_32)
        elif self.mode == "AMD64":
            self.cs = capstone.Cs(capstone.CS_ARCH_X86, capstone.CS_MODE_64)
        elif self.mode == "MIPS":
            self.cs = capstone.Cs(capstone.CS_ARCH_MIPS, capstone.CS_MODE_32 +
                                  capstone.CS_MODE_BIG_ENDIAN)
        # This is not really supported yet.
        elif self.mode == "ARM":
            self.cs = capstone.Cs(capstone.CS_ARCH_ARM, capstone.CS_MODE_ARM)
        else:
            raise NotImplementedError(
                "No disassembler available for this arch.")

        self.cs.detail = True
        self.cs.skipdata_setup = ("db", None, None)
        self.cs.skipdata = True

    def disassemble(self, data, offset):
        for insn in self.cs.disasm(data, int(offset)):
            yield CapstoneInstruction(insn, session=self.session,
                                      address_space=self.address_space)


class Disassemble(plugin.Command):
    """Disassemble the given offset."""

    __name = "dis"

    @classmethod
    def args(cls, parser):
        super(Disassemble, cls).args(parser)
        parser.add_argument(
            "offset", type="SymbolAddress",
            help="An offset to disassemble. This can also be the name of "
            "a symbol with an optional offset. For example: "
            "tcpip!TcpCovetNetBufferList.")

        parser.add_argument("-a", "--address_space", default=None,
                            help="The address space to use.")

        parser.add_argument(
            "-l", "--length", type="IntParser",
            help="The number of instructions (lines) to disassemble.")

        parser.add_argument(
            "-e", "--end", default=None, type="IntParser",
            help="The end address to disassemble up to.")

        parser.add_argument(
            "--mode", default="auto", choices=["auto", "I386", "AMD64", "MIPS"],
            type="Choices",
            help="Disassemble Mode (AMD64 or I386). Defaults to 'auto'.")

        parser.add_argument(
            "--suppress_headers", default=False, type="Boolean",
            help="If set we do not write table headers.")

        parser.add_argument(
            "--branch", default=False, type="Boolean",
            help="If set we follow all branches to cover all code.")

        parser.add_argument(
            "--canonical", default=False, type="Boolean",
            help="If set emit canonical instructions. These can be used to "
            "develop signatures.")

    def __init__(self, offset=0, address_space=None, length=None,
                 end=None, mode="auto", suppress_headers=False, branch=False,
                 canonical=False, **kwargs):
        super(Disassemble, self).__init__(**kwargs)

        load_as = self.session.plugins.load_as(session=self.session)
        self.address_space = load_as.ResolveAddressSpace(address_space)
        self.canonical = canonical

        resolver = self.session.address_resolver
        offset = resolver.get_address_by_name(offset)
        if end:
            end = resolver.get_address_by_name(end)

        # Normalize the offset to an address.
        offset = obj.Pointer.integer_to_address(offset)

        self.offset = offset
        self.length = length
        self.end = end

        # If length is not specified only disassemble one pager of output.
        if self.length is None:
            self.length = self.session.GetParameter("paging_limit", 50)

        # If end is specified, keep going until we hit the end.
        if self.end is not None:
            self.length = 2**62

        # All the visited addresses (for branch analysis).
        self._visited = set()
        self._visited_count = 0

        self.follow_branches = branch
        self.suppress_headers = suppress_headers
        if mode == "auto":
            mode = None

        self.mode = mode
        self.func = Function(offset=offset, vm=self.address_space,
                             session=self.session, mode=self.mode)

    def disassemble(self, offset, depth=0):
        """Disassemble the number of instructions required.

        Yields:
          A tuple of (Address, Opcode, Instructions).
        """
        # Disassemble the data one page at the time.
        func = Function(offset=offset, vm=self.address_space,
                        session=self.session, mode=self.mode)

        for instruction in func.disassemble(self.length):
            offset = instruction.address

            if offset in self._visited:
                return

            self._visited_count += 1

            # Exit condition can be specified by length.
            if (self.length is not None and
                    self._visited_count > self.length):
                return

            # Exit condition can be specified by end address.
            if self.end and offset > self.end:
                return

            # Yield this data.
            yield depth, instruction

            if self.follow_branches:
                self._visited.add(offset)

            # If the user asked for full branch analysis we follow all
            # branches. This gives us full code coverage for a function - we
            # just disassemble until the function exists from all branches.
            if self.follow_branches:
                # A return stops this branch.
                if instruction.is_return():
                    return

                target = instruction.target
                if target:
                    # Start disassembling the branch. When the branch is
                    # exhausted we resume disassembling the continued
                    # branch.
                    for x in self.disassemble(target, depth=depth+1):
                        yield x

                    # A JMP stops disassembling this branch. This happens with
                    # tail end optimization where a JMP would meet a RET which
                    # unwinds past the JMP.
                    if instruction.mnemonic.startswith("jmp"):
                        return

    def render_canonical(self, renderer):
        """Renders a canonical description of each instruction.

        Canonical descriptions are machine readable representations of the
        instruction which can be used to write disassembler signatures.
        """
        # If length nor end are specified only disassemble one pager output.
        if self.end is None and self.length is None:
            self.length = self.session.GetParameter("paging_limit") - 5

        renderer.table_header([
            ('Instruction', "instruction", ''),
        ], suppress_headers=True)

        for _, instruction in self.disassemble(self.offset):
            renderer.table_row(instruction.GetCanonical())

    def render(self, renderer):
        """Disassemble code at a given address.

        Disassembles code starting at address for a number of bytes
        given by the length parameter (default: 128).

        Note: This feature requires capstone, available at
            http://www.capstone-engine.org/

        The mode is '32bit' or '64bit'. If not supplied, the disassembler
        mode is taken from the profile.
        """
        if self.canonical:
            return self.render_canonical(renderer)

        # If we are doing branch analysis we can not suspend this plugin. We
        # must do everything all the time.
        if self.follow_branches:
            self.length = self.end = None

        renderer.table_header(
            [dict(type="TreeNode", name="Address", cname="cmd_address",
                  child=dict(style="address")),
             dict(name='Rel', cname="relative_address", style="address",
                  width=5),
             ('Op Codes', "opcode", '<20'),
             ('Instruction', "instruction", '<40'),
             ('Comment', "comment", "")],
            suppress_headers=self.suppress_headers)

        self._visited.clear()
        self._visited_count = 0

        for depth, instruction in self.disassemble(self.offset):
            offset = instruction.address

            relative = None
            resolver = self.session.address_resolver
            if resolver:
                (f_offset, f_names) = resolver.get_nearest_constant_by_address(
                    offset)

                f_name = ", ".join(f_names)
                self.session.report_progress(
                    "Disassembled %s: 0x%x", f_name, offset)

                if offset - f_offset == 0:
                    renderer.table_row("------ %s ------\n" % f_name,
                                       annotation=True)

                if offset - f_offset < 0x1000:
                    relative = offset - f_offset

            renderer.table_row(
                instruction.address, relative, instruction.hexbytes,
                instruction.op_str, instruction.comment, depth=depth)

        # Continue from where we left off when the user calls us again with the
        # v() plugin.
        self.offset = offset


class TestDisassemble(testlib.SimpleTestCase):
    PARAMETERS = dict(
        # We want to test symbol discovery via export table detection so turn it
        # on.
        commandline=("dis -l %(length)s %(func)s "
                     "--name_resolution_strategies Export"),
        func=0x805031be,
        length=20
        )


class Function(obj.BaseAddressComparisonMixIn, obj.BaseObject):
    """A base object representing code snippets."""

    def __init__(self, mode=None, args=None, **kwargs):
        super(Function, self).__init__(**kwargs)
        self.args = args
        if mode is None:
            mode = self.obj_context.get("mode")

        if mode is None:
            # Autodetect disassembling mode
            highest_usermode_address = self.obj_session.GetParameter(
                "highest_usermode_address")

            # We are disassembling user space.
            if self.obj_offset < highest_usermode_address:
                mode = self.obj_session.GetParameter(
                    "process_context").address_mode

        # fall back to the kernel's mode.
        if not mode:
            mode = self.obj_session.profile.metadata("arch") or "I386"

        self.dis = Capstone(mode, address_space=self.obj_vm,
                            session=self.obj_session)
        self.mode = mode

    def __int__(self):
        return self.obj_offset

    def __hash__(self):
        return self.obj_offset + hash(self.obj_vm)

    def __unicode__(self):
        if self.mode == "AMD64":
            format_string = "%0#14x  %s"
        else:
            format_string = "%0#10x  %s"

        result = []
        for instruction in self.disassemble():
            result.append(format_string % (
                instruction.address, instruction.text))

        return "\n".join(result)

    def __iter__(self):
        return iter(self.disassemble())

    def __getitem__(self, item):
        for i, x in enumerate(self.disassemble()):
            if i == item:
                return x

    def Rewind(self, length=0, align=True):
        """Returns another function which starts before this function.

        If align is specified, we increase the length repeatedly until the
        new function disassebles exactly to the same offset of this
        function.
        """
        while 1:
            offset = self.obj_offset - length
            result = self.obj_profile.Function(vm=self.obj_vm, offset=offset)
            if not align:
                return result

            for instruction in result.disassemble(instructions=length):
                # An exact match.
                if instruction.address == self.obj_offset:
                    return result

                # We overshot ourselves, try again.
                if instruction.address > self.obj_offset:
                    length += 1
                    break

    def disassemble(self, instructions=10):
        """Generate some instructions."""
        count = 0
        buffer_offset = offset = self.obj_offset
        while 1:
            # By default read 2 pages.
            data = self.obj_vm.read(buffer_offset, 0x2000)

            for instruction in self.dis.disassemble(data, buffer_offset):
                offset = instruction.address

                # If we disassemble past one page, we read another two
                # pages. This guarantees that we have enough data for full
                # instructions.
                if offset - buffer_offset > 0x1000:
                    buffer_offset = offset
                    break

                yield instruction
                count += 1

                if count > instructions:
                    return

            buffer_offset = offset


# Register the Function class in all profiles.
obj.Profile.COMMON_CLASSES["Function"] = Function



#!/usr/bin/env python2

# Rekall Memory Forensics
# Copyright 2015 Google Inc. All Rights Reserved.
#
# Author: Michael Cohen scudette@google.com
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or (at
# your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
#
from rekall import addrspace
from rekall import plugin
from rekall.plugins.overlays.windows import pe_vtypes

from rekall.plugins.windows import common


class LoadWindowsProfile(common.AbstractWindowsCommandPlugin):
    """Loads the profile into the session.

    If the profile does not exist in the repositories, fetch and build it from
    the symbol server. This plugin allows the user to change resolution of
    selected binaries by forcing the fetching of symbol files from the symbol
    server interactively.
    """

    name = "load_profile"

    interactive = True

    @classmethod
    def args(cls, parser):
        super(LoadWindowsProfile, cls).args(parser)
        parser.add_argument(
            "module_name",
            help="The name of the module (without the .pdb extensilon).",
            required=True)

        parser.add_argument(
            "guid", help="The guid of the module.",
            required=False)

    def __init__(self, module_name=None, guid=None, **kwargs):
        super(LoadWindowsProfile, self).__init__(**kwargs)
        self.module_name = module_name
        self.guid = guid

    def render(self, renderer):
        if self.guid is None:
            # Try to detect the GUID automatically.
            module = self.session.address_resolver.GetModuleByName(
                self.module_name)
            if not module:
                raise plugin.PluginError(
                    "Unknown module %s." % self.module_name)

            profile_name = module.detect_profile_name()
            if not profile_name:
                raise plugin.PluginError(
                    "Unable to determine GUID for module %s." %
                    self.module_name)
        else:
            profile_name = "%s/GUID/%s" % (self.module_name, self.guid)

        profile = self.session.LoadProfile(profile_name)
        if profile == None:
            # Try to build it from the symbol serv
            profile = module.build_local_profile(profile_name, force=True)
            if profile == None:
                raise plugin.PluginError(
                    "Unable to fetch or build %s" % profile_name)

        if profile:
            module.profile = profile



#!/usr/bin/env python2

# Rekall Memory Forensics
# Copyright 2015 Google Inc. All Rights Reserved.
#
# Author: Michael Cohen scudette@google.com
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or (at
# your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
#

__author__ = "Michael Cohen <scudette@google.com>"

"""A plugin to install relevant kernel modules to enable live analysis.

The intention is to allow the user to launch:

rekall live

and have Rekall install the right kernel module and connect to the driver on all
supported operating systems.
"""
import os
import platform
import re
import win32service

from rekall import resources
from rekall import plugin
from rekall.plugins.addrspaces import win32


class Live(plugin.ProfileCommand):
    """Launch a Rekall shell for live analysis on the current system."""

    name = "live"

    PROFILE_REQUIRED = False

    @classmethod
    def args(cls, parser):
        parser.add_argument("--driver", default=None,
                            help="Driver file to load")

        parser.add_argument("--device", default=r"\\.\pmem",
                            help="Device name to use")

        parser.add_argument("--service_name", default=r"pmem",
                            help="Service name to use")

    def __init__(self, driver=None, device=r"\\.\pmem", service_name="pmem",
                 **kw):
        super(Live, self).__init__(**kw)
        # It is OK for non privileged sessions to use the default drivers.
        if not self.session.privileged and driver:
            raise plugin.PluginError(
                "Installing arbitrary drivers is only available for "
                "interactive or privileged sessions.")

        self.driver = driver
        self.device = device
        self.name = service_name
        # Did we start the service? If so we need to clean up.
        self.we_started_service = False
        self.hScm = None
        self.hSvc = None

    exception_format_regex = re.compile(r": \((\d+),")
    def parse_exception(self, e):
        """Yes! seriously there is no way to get at the real error code."""
        # We often see code like if "Access Denied" in str(e):... but
        # this is unreliable since the message will be different when
        # different language packs are used. The best way is to
        # compare the numeric error code, but this is not exported. In
        # our testing e.errno is not being set properly by pywin32.
        m = self.exception_format_regex.search(str(e))
        if m:
            return int(m.group(1))

    def load_driver(self):
        """Load the driver if possible."""
        # Check the driver is somewhere accessible.
        if self.driver is None:
            # Valid values
            # http://superuser.com/questions/305901/possible-values-of-processor-architecture
            machine = platform.machine()
            if machine == "AMD64":
                driver = "winpmem_x64.sys"
            elif machine == "x86":
                driver = "winpmem_x86.sys"
            else:
                raise plugin.PluginError("Unsupported architecture")

            self.driver = resources.get_resource("WinPmem/%s" % driver)

            # Try the local directory
            if self.driver is None:
                self.driver = os.path.join(os.getcwd(), "WinPmem", driver)

        self.session.logging.debug("Loading driver from %s", self.driver)

        if not os.access(self.driver, os.R_OK):
            raise plugin.PluginError(
                "Driver file %s is not accessible." % self.driver)

        self.hScm = win32service.OpenSCManager(
            None, None, win32service.SC_MANAGER_CREATE_SERVICE)

        # First uninstall the driver
        self.remove_service(also_close_as=False)

        try:
            self.hSvc = win32service.CreateService(
                self.hScm, self.name, self.name,
                win32service.SERVICE_ALL_ACCESS,
                win32service.SERVICE_KERNEL_DRIVER,
                win32service.SERVICE_DEMAND_START,
                win32service.SERVICE_ERROR_IGNORE,
                self.driver,
                None, 0, None, None, None)

            self.session.logging.debug("Created service %s", self.name)
            # Remember to cleanup afterwards.
            self.we_started_service = True

        except win32service.error as e:
            # Service is already there, try to open it instead.
            self.hSvc = win32service.OpenService(
                self.hScm, self.name,
                win32service.SERVICE_ALL_ACCESS)

        # Make sure the service is stopped.
        try:
            win32service.ControlService(
                self.hSvc, win32service.SERVICE_CONTROL_STOP)
        except win32service.error:
            pass

        try:
            win32service.StartService(self.hSvc, [])
        except win32service.error, e:
            self.session.logging.debug("%s: will try to continue", e)

        try:
            return win32.WinPmemAddressSpace(
                session=self.session, filename=self.device)
        except IOError as e:
            raise plugin.PluginError(*e.args)

    def live(self):
        try:
            phys_as = win32.WinPmemAddressSpace(
                session=self.session, filename=self.device)
        except IOError as e:
            self.session.logging.debug("%s", e)
            errno = self.parse_exception(e)
            if errno == 5:   # Access Denied.
                raise plugin.PluginError(
                    "%s. Are you running as Administrator?" % e)

            elif errno == 2: # File not found
                phys_as = self.load_driver()

            else:
                raise plugin.PluginError("%s" % e)

        self.session.physical_address_space = phys_as

    def remove_service(self, also_close_as=True):
        self.session.logging.debug("Removing service %s", self.name)

        # Make sure the handle is closed.
        if also_close_as:
            self.session.physical_address_space.close()

        # Stop the service if it's running.
        if not self.hSvc:
            try:
                self.hSvc = win32service.OpenService(
                    self.hScm, self.name,
                    win32service.SERVICE_ALL_ACCESS)
            except win32service.error:
                self.session.logging.debug("%s service does not exist.",
                                           self.name)

        if self.hSvc:
            self.session.logging.debug("Stopping service: %s", self.name)
            try:
                win32service.ControlService(
                    self.hSvc, win32service.SERVICE_CONTROL_STOP)
            except win32service.error as e:
                self.session.logging.debug("Error stopping service: %s", e)

            self.session.logging.debug("Deleting service: %s", self.name)
            try:
                win32service.DeleteService(self.hSvc)
            except win32service.error as e:
                self.session.logging.debug("Error deleting service: %s", e)

            win32service.CloseServiceHandle(self.hSvc)

    def close(self):
        if self.we_started_service:
            self.remove_service()

    def __enter__(self):
        self.live()
        return self

    def __exit__(self, exc_type, exc_value, trace):
        self.close()

    def render(self, renderer):
        renderer.format("Launching live memory analysis\n")
        try:
            self.live()

            # Launch the shell.
            shell = self.session.plugins.shell()
            shell.render(renderer)
        finally:
            self.close()
